#!/usr/bin/env python

__author__ = "Xiang Gao, Huaiying Lin, Qunfeng Dong"
__copyright__ = "Copyright 2016, Bayesian-based LCA taxonomic classification"
__credits__ = ["Xiang Gao", "Huaiying Lin", "Qunfeng Dong"]
__license__ = "GPL"
__version__ = "1.2"
__maintainer__ = "Huaiying Lin"
__email__ = "hlin2@luc.edu"
__status__ = "Python3"

import sys
import os
import subprocess
import argparse

try:
    from Bio import AlignIO,SeqIO
except ImportError:
    sys.stderr.write("Error! BioPython is not detected!\n")
    sys.exit(1)

'''
Get a subset of lineage information from GreenGene Microbial database.
'''

##### parser ######

parser = argparse.ArgumentParser(description=
                                 ''' << Bayesian-based LCA taxonomic classification method >>\n\n   Please make sure the following softwares are in your PATH:
                                     1.muscle (http://www.drive5.com/muscle/downloads.htm), muscle should be the program's name.
                                     2.ncbi-blast suite (ftp://ftp.ncbi.nlm.nih.gov/blast/executables/blast+/LATEST/)
                                     3.clustalo (http://www.clustal.org/omega/), clustalo should be the program's name.
                                     4.Biopython should be installed locally.
                            
                                     This is the utility script to format Greengene Database before running the BLCA taxonomy profiling.
                                     >> Please first download the Greengenes fasta and taxonomy files from https://greengenes.secondgenome.com/?prefix=downloads/greengenes_database/gg_13_5/.''',
                                 epilog="No warrenty comes with this script. Author: hlin2@luc.edu. \nAny suggestions or bugs report are welcomed.",
                                 add_help=False, formatter_class=argparse.RawTextHelpFormatter)
##### Other arguments #####
optional = parser.add_argument_group('optional arguments')
optional.add_argument("--dir", default='gg',
                      help="The local directory name where you want to store the formatted database. Default: gg",
                      type=str)
optional.add_argument("--ggfasta", default='gg_13_5.fasta.gz',
                      help="The GreenGene database fasta file. Default: gg_13_5.fasta.gz", type=str)
optional.add_argument("--ggtax", default='gg_13_5_taxonomy.txt.gz',
                      help="The GreenGene database taxonomy file. Default: gg_13_5_taxonomy.txt.gz", type=str)
optional.add_argument("-t","--fulltax", action='store_true',
                      help="Extract a subset of GreenGene with only reads with full taxonomy information. This could take a while.")
optional.add_argument("-h", "--help", help="show this help message and exit", action="help")
##### parse arguments #####
args = parser.parse_args()

cwd = os.getcwd()
args.ggfasta = os.path.join(cwd, args.ggfasta)
args.ggtax = os.path.join(cwd, args.ggtax)
args.dir = os.path.join(cwd, args.dir)

################ Function ##############

def check_program(prgname):
    '''Check whether a program has been installed and put in the PATH'''
    path = os.popen("which " + prgname).read().rstrip()
    if len(path) > 0 and os.path.exists(path):
        print(prgname + " is located in your PATH!")
    else:
        print("ERROR: ", prgname, " is NOT in your PATH, please set up ", "!")
        sys.exit(1)

def format_ext_gg_taxfile(taxfile):
    ''' Read in Greengenes taxonomy file, and format it and write out into BLCA compatiable format '''
    leveldic = {'c': 'class', 'g': 'genus', 'k': 'superkingdom', 'f': 'family', 'o': 'order', 'p': 'phylum',
                's': 'species'}
    taxin = open(taxfile)
    fsqName = []
    outtaxfile = os.path.splitext(taxfile)[0] + ".fullTax.taxonomy"
    if os.path.isfile(outtaxfile):
        os.remove(outtaxfile)
    taxout=open(outtaxfile,'w')
    for l in taxin:
        ln = l.rstrip().replace(" ", "").split("\t")
        if len(ln) == 2:
            tmp = dict(x.split("__") for x in ln[1].split(";"))
            naN = sum([ tmp[k] == "" for k in tmp ])
            if naN == 0:
                fsqName.append(ln[0])
                taxout.write(ln[0] + "\t")
                oldnames = list(tmp.keys())
                for k in oldnames:
                    tmp[leveldic[k]] = tmp.pop(k)
                for key, val in tmp.items():
                    taxout.write(key + ':' + val + ";")
                taxout.write("\n")
    taxin.close()
    taxout.close()
    print(os.path.basename(taxfile), "has been formatted and outputted!")
    return(fsqName)

def format_gg_taxfile(taxfile):
    ''' Read in Greengenes taxonomy file, and format it and write out into BLCA compatiable format '''
    leveldic={'c':'class','g':'genus','k':'superkingdom','f':'family','o':'order','p':'phylum','s':'species'}
    taxin=open(taxfile)
    outtaxfile = os.path.splitext(taxfile)[0] + ".taxonomy"
    if os.path.isfile(outtaxfile):
        os.remove(outtaxfile)
    taxout=open(outtaxfile,'w')
    for l in taxin:
        ln=l.rstrip().replace(" ","").split("\t")
        if len(ln) == 2:
            taxout.write(ln[0] + "\t")
            tmp = dict(x.split("__") for x in ln[1].split(";"))
            oldnames = list(tmp.keys())
            for k in oldnames:
                tmp[leveldic[k]]=tmp.pop(k)
            for key, val in tmp.items():
                taxout.write(key+':'+val+";")
            taxout.write("\n")
    taxin.close()
    taxout.close()
    print(os.path.basename(taxfile), "has been formatted and outputted!")

def extract_faseq(fsa, seqlist):
    ''' Extract seqs according to taxonomy info'''
    outfsa = os.path.splitext(fsa)[0] + ".fullTax.fasta"
    outhandler = open(outfsa,"w")
    record_dict = SeqIO.index(fsa, "fasta")
    for ID in seqlist:
        SeqIO.write(record_dict[ID], outhandler, "fasta")
    outhandler.close()
    print("Full taxonomy sequences have been extracted from",fsa,"!")
    return(outfsa)

def make_blastdb(dbfsa):
    ''' Format downloaded Greengenes sequence fasta files into blast compatible format '''
    check_program('makeblastdb')
    dbname = os.path.splitext(dbfsa)[0]
    subprocess.call(['makeblastdb', '-dbtype', "nucl", '-in', dbfsa, '-parse_seqids', '-out', dbname])
    print(os.path.basename(dbfsa), "has been formatted!")

def ungz(file, folder):
    fname = os.path.splitext(os.path.basename(file))[0]
    os.system("gzip -dc " + file + " > " + folder + "/" + fname)
    print(file, "has be unzipped!")
    return(os.path.join(folder,fname))


######## MAIN ###########

if __name__ == "__main__":
    os.system('rm -fr ' + args.dir)
    os.system('mkdir ' + args.dir)

    dbfsa = ungz(args.ggfasta, args.dir)
    taxfile = ungz(args.ggtax, args.dir)

    if args.fulltax:
        validseqs = format_ext_gg_taxfile(taxfile)
        newdbfsa = extract_faseq(dbfsa,validseqs)
        make_blastdb(newdbfsa)
    else:
        format_gg_taxfile(taxfile)
        make_blastdb(dbfsa)




# for i in range(1, 73):
#
#     cmd = 'srun -n 1 python3 00_DataNeeded/BLCA/2.blca_main.py -x -p 1 -r 00_DataNeeded/db_GTDB214.1/ssu_all_r214_BLCAparsed.taxonomy -q 00_DataNeeded/db_GTDB214.1/ssu_all_r214_BLCAparsed.fasta -i b_rep_set_%s.fa -o b_rep_set_%s_BLCA_out.1.txt &' % (i, i)
#     print(cmd)


def ko_stats_to_txt(ko_stats_dict, ko_desc_dict, op_txt, op_stats_txt):
    op_txt_handle = open(op_txt, 'w')
    op_stats_txt_handle = open(op_stats_txt, 'w')
    for ko_high in sorted(list(ko_stats_dict.keys())):
        ko_d_set = ko_stats_dict[ko_high]
        ko_high_desc = ko_desc_dict[ko_high]
        op_stats_txt_handle.write('%s\t%s\t%s\n' % (ko_high, len(ko_d_set), ko_high_desc))
        for ko_d in sorted(list(ko_d_set)):
            ko_d_desc = ko_desc_dict[ko_d]
            op_txt_handle.write('%s\t%s\t%s\t%s\n' % (ko_high, ko_high_desc, ko_d, ko_d_desc))
    op_txt_handle.close()
    op_stats_txt_handle.close()

print('aaa')




import os
import glob
from Bio import SeqIO


file_list   = glob.glob('b_rep_set_splitted/*.fa')
blastn_dir  = 'blastn_splitted'

seq_to_file_dict = dict()
for each_file in file_list:
    for each_seq in SeqIO.parse(each_file, 'fasta'):
        seq_id = each_seq.id
        seq_to_file_dict[seq_id] = os.path.basename(each_file)


dict = dict()
for each_line in open('b_rep_set.fasta.blastn'):

    seq_id = each_line.split('\t')[0]
    fa_file_name = seq_to_file_dict[seq_id]
    blastn_file = '%s/%s.blastn' % (blastn_dir, fa_file_name)

    if blastn_file not in dict:
        dict[blastn_file] = []

    dict[blastn_file].append(each_line.strip())


for each_b_file in dict:
    lines = dict[each_b_file]
    each_b_file_handle = open(each_b_file, 'w')
    each_b_file_handle.write('\n'.join(lines))
    each_b_file_handle.close()




import os
import argparse
from Bio import SeqIO

bam2reads_usage = '''
=========================== bam2reads example commands ===========================

module load samtools/1.15
BioSAK bam2reads -b sorted.bam -o bam2reads_wd_ctg1 -r ctg_1
BioSAK bam2reads -b sorted.bam -o bam2reads_wd_ctg2 -r ctg_2:200-5000
BioSAK bam2reads -b sorted.bam -o bam2reads_wd_ctg3 -r ctg_3:200-
BioSAK bam2reads -b sorted.bam -o bam2reads_wd_ctg4 -r ctg_4:-500 -s reads.fastq

==================================================================================
'''


def select_seq(args):

    # read in argument
    seq_file      = args['seq']
    id_file       = args['id']
    select_option = args['option']
    output_file   = args['out']
    one_line      = args['oneline']
    in_fastq      = args['fq']

    # get provided id list
    seq_id_list = set()
    for seq_id in open(id_file):
        seq_id_list.add(seq_id.strip())

    seq_in_format = 'fasta'
    if in_fastq is True:
        seq_in_format = 'fastq'

    # extract sequences
    output_file_handle = open(output_file, 'w')
    for seq_record in SeqIO.parse(seq_file, seq_in_format):
        seq_id = seq_record.id
        if select_option == 1:
            if seq_id in seq_id_list:

                if in_fastq is False:
                    if one_line is False:
                        SeqIO.write(seq_record, output_file_handle, 'fasta')
                    else:
                        SeqIO.write(seq_record, output_file_handle, 'fasta-2line')
                else:
                    SeqIO.write(seq_record, output_file_handle, 'fastq')
        if select_option == 0:
            if seq_id not in seq_id_list:

                if in_fastq is False:
                    if one_line is False:
                        SeqIO.write(seq_record, output_file_handle, 'fasta')
                    else:
                        SeqIO.write(seq_record, output_file_handle, 'fasta-2line')
                else:
                    SeqIO.write(seq_record, output_file_handle, 'fastq')
    output_file_handle.close()


def bam2reads(args):

    bam_file        = args['b']
    region_str      = args['r']
    read_seq_file   = args['s']
    output_dir      = args['o']
    force_overwrite = args['force']

    # create output folder
    if os.path.isdir(output_dir) is True:
        if force_overwrite is False:
            print('Output folder detected, program exited!')
            exit()
        else:
            os.system('rm -r %s' % output_dir)
    os.system('mkdir %s' % output_dir)

    in_fastq = False
    if read_seq_file is not None:
        read_seq_file_ext = read_seq_file.split('.')[-1]
        if 'q' in read_seq_file_ext:
            in_fastq = True

    # file out
    mpileup_positions_file  = '%s/mpileup_positions.txt' % output_dir
    mpileup_op_file         = '%s/bam.mpileup'           % output_dir
    bam_header_txt          = '%s/bam_header.txt'        % output_dir
    read_id_txt             = '%s/reads_id.txt'          % output_dir
    extracted_read_file     = '%s/extracted_reads.fasta' % output_dir
    if in_fastq is True:
        extracted_read_file = '%s/extracted_reads.fastq' % output_dir

    # get bam header
    get_bam_header_cmd = 'samtools view -H %s > %s' % (bam_file, bam_header_txt)
    print('Running %s' % get_bam_header_cmd)
    os.system(get_bam_header_cmd)

    # get ref_seq_len_dict
    ref_seq_len_dict = {}
    for each_line in open(bam_header_txt):
        each_line_split = each_line.strip().split('\t')
        if each_line.startswith('@'):
            ref_seq_id = ''
            ref_seq_len = 0
            for each_element in each_line_split:
                if each_element.startswith('SN:'):
                    ref_seq_id = each_element[3:]
                if each_element.startswith('LN:'):
                    ref_seq_len = int(each_element[3:])
            ref_seq_len_dict[ref_seq_id] = ref_seq_len

    # get region_list
    seq_id = region_str
    region_list = []
    if ':' in region_str:
        region_str_split = region_str.split(':')
        seq_id = region_str_split[0]
        region_list_original = region_str_split[1].split('-')
        ref_seq_len = ref_seq_len_dict[seq_id]

        # update left side pos
        if region_list_original[0] == '':
            region_list.append(1)
        else:
            region_list.append(int(region_list_original[0]))

        # update right side pos
        if region_list_original[1] == '':
            region_list.append(ref_seq_len)
        else:
            if int(region_list_original[1]) <= ref_seq_len:
                region_list.append(int(region_list_original[1]))
            else:
                region_list.append(ref_seq_len)
    else:
        region_list = [1, ref_seq_len_dict[seq_id]]

    # write out mpileup_positions_file
    mpileup_positions_file_handle = open(mpileup_positions_file, 'w')
    for each_pos in range(region_list[0], (region_list[1] + 1)):
        mpileup_positions_file_handle.write('%s\t%s\n' % (seq_id, each_pos))
    mpileup_positions_file_handle.close()

    # run samtools mpileup
    mpileup_cmd = 'samtools mpileup -Q 0 -l %s --output-QNAME -o %s %s' % (mpileup_positions_file, mpileup_op_file, bam_file)
    print('Running %s' % mpileup_cmd)
    os.system(mpileup_cmd)

    # parse samtools mpileup output
    print('Parsing mpileup output')
    aligned_read_set = set()
    for each_line in open(mpileup_op_file):
        each_line_split = each_line.strip().split('\t')
        current_aligned_read_list = each_line_split[6].split(',')
        for each_read in current_aligned_read_list:
            aligned_read_set.add(each_read)

    # write out read id
    read_id_txt_handle = open(read_id_txt, 'w')
    for each_read in aligned_read_set:
        read_id_txt_handle.write(each_read + '\n')
    read_id_txt_handle.close()
    print('ID of %s reads aligned to %s were exported to: %s' % (len(aligned_read_set), region_str, read_id_txt))

    # extract sequences
    if read_seq_file is not None:
        select_seq_arg_dict = {'seq': read_seq_file,  'id': read_id_txt,  'option': 1, 'out': extracted_read_file,  'oneline': True,  'fq': in_fastq}
        select_seq(select_seq_arg_dict)
    print('Sequence of %s reads aligned to %s were exported to: %s' % (len(aligned_read_set), region_str, extracted_read_file))
    print('Done!')


if __name__ == '__main__':

    parser = argparse.ArgumentParser(usage=bam2reads_usage)
    parser.add_argument('-b',     required=True,                       help='Sorted Bam file')
    parser.add_argument('-r',     required=True,                       help='Interested region')
    parser.add_argument('-s',     required=False, default=None,        help='Read sequence file')
    parser.add_argument('-o',     required=True,                       help='Output folder')
    parser.add_argument('-force', required=False, action="store_true", help='Force overwriting')
    args = vars(parser.parse_args())
    bam2reads(args)



import os
import glob
import argparse
import multiprocessing as mp
from distutils.spawn import find_executable


get_genome_NCBI_parser_usage = '''
===================================== get_genome_NCBI example commands =====================================

# Dependency: datasets (https://www.ncbi.nlm.nih.gov/datasets/docs/v1/download-and-install/)

# example command
BioSAK get_genome_NCBI -i GCA_000006945
BioSAK get_genome_NCBI -i GCA_009840555.1
BioSAK get_genome_NCBI -i genome_id.txt -o downloaded_gnm_dir -t 8 -f

# Format of genome_id.txt
GCA_009837245
GCA_009840555.1
GCA_000006945.2

# You can use this link to get the id of genomes from a specific taxon , IDs are in the "Assembly" column.
https://www.ncbi.nlm.nih.gov/genome/browse#!/prokaryotes/refseq_category:reference

============================================================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def datasets_worker(arg_list):

    gnm_id                      = arg_list[0]
    op_dir                      = arg_list[1]
    downloaded_file             = '%s/%s.zip'                                                                               % (op_dir, gnm_id)
    downloaded_file_unzip_dir   = '%s/%s'                                                                                   % (op_dir, gnm_id)
    datasets_cmd                = 'datasets download genome accession %s --include genome --no-progressbar --filename %s'   % (gnm_id, downloaded_file)
    os.system(datasets_cmd)

    # datasets download genome accession %s --include genome --no-progressbar --filename %s
    # datasets download genome accession GCA_014843695.1 --include gff3,rna,cds,protein,genome,seq-report

    if os.path.isfile(downloaded_file) is True:

        if os.path.isdir(downloaded_file_unzip_dir) is True:
            os.system('rm -r %s' % downloaded_file_unzip_dir)

        unzip_cmd = 'unzip -q %s -d %s' % (downloaded_file, downloaded_file_unzip_dir)
        os.system(unzip_cmd)

        fna_file_re  = '%s/ncbi_dataset/data/*.*/*.*_genomic.fna' % (downloaded_file_unzip_dir)
        fna_file_list = glob.glob(fna_file_re)

        if len(fna_file_list) == 1:
            pwd_fna_file_unzipped = fna_file_list[0]

            _, f_base, _ = sep_path_basename_ext(pwd_fna_file_unzipped)
            pwd_fna_file = '%s/%s.fna' % (op_dir, '_'.join(f_base.split('_')[:2]))

            mv_cmd = 'mv %s %s'  % (pwd_fna_file_unzipped, pwd_fna_file)
            os.system(mv_cmd)
            os.system('rm %s'    % downloaded_file)
            os.system('rm -r %s' % downloaded_file_unzip_dir)
        else:
            print('Download failed: %s!' % gnm_id)
    else:
        print('Download failed: %s!' % gnm_id)


def download_GenBank_genome(args):

    assembly_id     = args['i']
    output_folder   = args['o']
    num_threads     = args['t']
    force_overwrite = args['f']

    if find_executable('datasets') is None:
        print('datasets not detected, please install it first')
        print('get help with: BioSAK get_genome_NCBI -h')
        exit()

    if os.path.isfile(assembly_id) is False:
        print('Download single genome mode')
        if output_folder is None:
            output_folder = '.'
        datasets_worker([assembly_id, output_folder])

    else:
        # check output dir
        if output_folder is None:
            print('An output directory need to be provided, program exited!')
            exit()
        else:
            if os.path.isdir(output_folder) is True:
                if force_overwrite is True:
                    os.system('rm -r %s' % output_folder)
                else:
                    print('Specified output directory detected, program exited!')
                    exit()
            os.mkdir(output_folder)

        report_file = '%s_report.txt'        % output_folder
        failed_file = '%s_report_failed.txt' % output_folder

        datasets_worker_arg_lol = []
        for each_gnm in open(assembly_id):
            gnm_id = each_gnm.strip()
            datasets_worker_arg_lol.append([gnm_id, output_folder])

        # run with multiprocessing
        pool = mp.Pool(processes=num_threads)
        pool.map(datasets_worker, datasets_worker_arg_lol)
        pool.close()
        pool.join()

        # report
        failed_gnm_set = set()
        id_to_file_dict = dict()
        report_file_handle = open(report_file, 'w')
        for each_gnm in open(assembly_id):
            gnm_id = each_gnm.strip()
            current_gnm_re = '%s/%s*.fna' % (output_folder, gnm_id)
            current_gnm_list = glob.glob(current_gnm_re)

            if len(current_gnm_list) == 1:
                pwd_current_gnm = current_gnm_list[0]
                id_to_file_dict[gnm_id] = pwd_current_gnm.split('/')[-1]
                report_file_handle.write('%s\t%s\n' % (gnm_id, pwd_current_gnm.split('/')[-1]))
            else:
                failed_gnm_set.add(gnm_id)
        report_file_handle.close()

        # write out failed files
        if len(failed_gnm_set) > 0:
            failed_file_handle = open(failed_file, 'w')
            failed_file_handle.write('\n'.join(failed_gnm_set))
            failed_file_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser(usage=get_genome_NCBI_parser_usage)
    parser.add_argument('-i',   required=True,                       help='IDs of genomes to download')
    parser.add_argument('-o',   required=False, default=None,        help='output folder')
    parser.add_argument('-t',   required=False, default=1, type=int, help='number of threads')
    parser.add_argument('-f',   required=False, action="store_true", help='force overwrite existing genome directory')
    args = vars(parser.parse_args())
    download_GenBank_genome(args)



import os
import argparse


hpc4_usage = '''
========================= hpc4 example commands =========================

BioSAK hpc4 -wt 119:59:59 -conda mybase2 -t 12 -a spongeholobiont -q amd -n iqtree -c "iqtree -h"
BioSAK hpc4 -wt 119:59:59 -conda mybase2 -t 12 -a spongeholobiont -q intel -n iqtree -c "iqtree -h"
BioSAK hpc4 -wt 119:59:59 -conda mybase2 -t 12 -a marmolecol -q gpu-a30 -tpc 3 -n mcmctree -c mcmctree_cmds.txt
BioSAK hpc4 -wt 119:59:59 -conda mybase2 -t 12 -a marmolecol -q gpu-l20 -n iqtree_2 -c "iqtree -h"

# To use srun, you commands must NOT contain the double quote symbol (").

# Max resources per User
intel       256 cores   2 nodes	    120h	10	15
amd         1536 cores  6 nodes	    120h	20	30
gpu-a30     16 gpu      4 nodes	    120h	4	8
gpu-l20     8 gpu       2 nodes	    120h	2	4
gpu-rtx5880 12 gpu      2 nodes	    120h	4	8

=========================================================================
'''


def hpc4(args):

    cmd                 = args['c']
    job_name            = args['n']
    email_address       = args['m']
    walltime            = args['wt']
    node_num            = args['node']
    core_num            = args['t']
    core_num_per_cmd    = args['tpc']
    queue_name          = args['q']
    conda_env           = args['conda']
    setting_a           = args['a']
    use_srun            = args['srun']

    js_file = '%s.sh' % job_name
    js_file_handle = open(js_file, 'w')
    js_file_handle.write('#!/bin/bash\n')

    if email_address is not None:
        js_file_handle.write('#SBATCH --mail-user=%s\n' % email_address)
        js_file_handle.write('#SBATCH --mail-type=begin\n')
        js_file_handle.write('#SBATCH --mail-type=end\n')

    js_file_handle.write('#SBATCH -t %s\n'          % walltime)
    js_file_handle.write('#SBATCH -N %s -n %s\n'    % (node_num, core_num))
    js_file_handle.write('#SBATCH -J %s\n'          % job_name)
    if queue_name in ['cpu', 'gpu', 'himem', 'amd', 'intel', 'gpu-a30', 'gpu-l20']:
        js_file_handle.write('#SBATCH -A %s\n'      % setting_a)
    js_file_handle.write('#SBATCH -p %s\n\n'        % queue_name)
    js_file_handle.write('cd $SLURM_SUBMIT_DIR\n\n')

    if conda_env is not None:
        js_file_handle.write('source ~/.bashrc\nconda activate %s\n'  % conda_env)

    if os.path.isfile(cmd) is False:
        js_file_handle.write('%s\n' % cmd)
    else:
        # write out to job script using srun
        cmd_list = []
        for each_cmd in open(cmd):
            if len(each_cmd.strip()) > 0:
                cmd_list.append(each_cmd.strip())

        for each_cmd in cmd_list[:-1]:
            if use_srun is False:
                js_file_handle.write('srun -n %s %s &\n' % (core_num_per_cmd, each_cmd))
            else:
                js_file_handle.write('srun -n %s BioSAK srun -c "%s" &\n' % (core_num_per_cmd, each_cmd))

        if use_srun is False:
            js_file_handle.write('srun -n %s %s\n' % (core_num_per_cmd, cmd_list[-1]))
        else:
            js_file_handle.write('srun -n %s BioSAK srun -c "%s"\n' % (core_num_per_cmd, cmd_list[-1]))

        js_file_handle.write('wait\n')

    js_file_handle.close()

    # submit jobscript
    os.system('sbatch %s' % js_file)


if __name__ == '__main__':

    hpc4_parser = argparse.ArgumentParser(usage=hpc4_usage)
    hpc4_parser.add_argument('-c',        required=True,                          help='command to submit')
    hpc4_parser.add_argument('-n',        required=True,                          help='job name')
    hpc4_parser.add_argument('-m',        required=False, default=None,           help='email address')
    hpc4_parser.add_argument('-wt',       required=False, default='23:59:59',     help='walltime, default: 23:59:59')
    hpc4_parser.add_argument('-node',     required=False, type=int, default=1,    help='number of node, default: 1')
    hpc4_parser.add_argument('-t',        required=False, type=int, default=12,   help='number of core, default: 12')
    hpc4_parser.add_argument('-tpc',      required=False, type=int, default=1,    help='number of core per command, default: 1')
    hpc4_parser.add_argument('-a',        required=False, default='boqianpy',     help='-A, boqianpy or oces, default: boqianpy')
    hpc4_parser.add_argument('-q',        required=True,                          help='queue, select from: cpu, cpu-share')
    hpc4_parser.add_argument('-conda',    required=False, default=None,           help='conda environment')
    hpc4_parser.add_argument('-srun',     required=False,  action='store_true',   help='use srun')
    args = vars(hpc4_parser.parse_args())
    hpc4(args)



import os
import argparse


cdb_usage = '''
===== cdb example commands =====

BioSAK cdb -i Cdb.csv

================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


def cdb(args):

    Cdb_file = args['i']
    cdb_name, cdb_path, cdb_base, cdb_ext = sep_path_basename_ext(Cdb_file)

    op_file_1 = '%s/%s_reformatted_1.%s' % (cdb_path, cdb_base, cdb_ext)
    op_file_2 = '%s/%s_reformatted_2.%s' % (cdb_path, cdb_base, cdb_ext)

    bin_to_cluster_dict = {}
    cluster_to_bin_dict = {}
    obtained_clusters = set()
    col_index = dict()
    line_num_index = 0
    for each_line in open(Cdb_file):
        line_num_index += 1
        line_split = each_line.strip().split(',')
        if line_num_index == 1:
            col_index = {key: i for i, key in enumerate(line_split)}
        else:
            bin_id = line_split[col_index['genome']]
            secondary_cluster = line_split[col_index['secondary_cluster']]
            obtained_clusters.add(secondary_cluster)
            bin_to_cluster_dict[bin_id] = secondary_cluster
            if secondary_cluster not in cluster_to_bin_dict:
                cluster_to_bin_dict[secondary_cluster] = [bin_id]
            else:
                cluster_to_bin_dict[secondary_cluster].append(bin_id)

    obtained_clusters_list = sorted([i for i in obtained_clusters])

    # write out 1
    op_file_1_handle = open(op_file_1, 'w')
    for j in obtained_clusters_list:
        op_file_1_handle.write('cluster_%s\t%s\n' % (j, '\t'.join(cluster_to_bin_dict[j])))
    op_file_1_handle.close()

    # write out 2
    op_file_2_handle = open(op_file_2, 'w')
    for j in obtained_clusters_list:
        cluster_gnm_set = cluster_to_bin_dict[j]
        for each_g in cluster_gnm_set:
            op_file_2_handle.write('%s\t%s\n' % (each_g, j))
    op_file_2_handle.close()


if __name__ == '__main__':
    cdb_parser = argparse.ArgumentParser(usage=cdb_usage)
    cdb_parser.add_argument('-i',  required=True, help='input file, Cdb.csv')
    args = vars(cdb_parser.parse_args())
    cdb(args)




import os
import glob
import argparse
from Bio import SeqIO
import multiprocessing as mp


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


def gbk2faa(gbk_in, faa_out):

    _, _, faa_base, _ = sep_path_basename_ext(faa_out)

    faa_out_handle = open(faa_out, 'w')
    for seq_record in SeqIO.parse(gbk_in, 'genbank'):
        for feature in seq_record.features:
            if feature.type == 'CDS':
                feature_translation = ''
                if 'translation' in feature.qualifiers:
                    feature_translation = feature.qualifiers['translation'][0]
                if feature_translation != '':
                    faa_out_handle.write('>%s__%s\n' % (faa_base, feature.qualifiers['protein_id'][0]))
                    faa_out_handle.write('%s\n' % feature_translation)
    faa_out_handle.close()



file_dir = '/Users/songweizhi/Desktop/Sponge_r226/sponge_phylogeny/COI/4110_wd/tmp'
file_ext = 'gbk'
file_re = '%s/*.%s' % (file_dir, file_ext)
file_list = glob.glob(file_re)


for file in sorted(file_list):
    print(file)
    faa_file = file.replace('.gbk', '.faa')
    gbk2faa(file, faa_file)












import os
import argparse


get_GTDB_taxon_gnm_parser_usage = '''
====================================== get_GTDB_taxon_gnm example commands ======================================

# Example commands
BioSAK get_GTDB_taxon_gnm -p Bac -meta bac120_metadata_r202.tsv -path genome_paths.tsv -taxon taxons.txt -cpl 85 -ctm 5
BioSAK get_GTDB_taxon_gnm -p Arc -meta ar122_metadata_r202.tsv -path genome_paths.tsv -taxon taxons.txt -cpl 50 -ctm 10

# -meta: bac120_metadata_r202.tsv or ar122_metadata_r202.tsv
# -path: genome_paths.tsv, in fastani folder from auxillary_files
# -taxon: one taxon per line, example below:
p__Thermoproteota
c__Methanosarcinia
f__Thalassarchaeaceae

=================================================================================================================
'''


def get_GTDB_taxon_gnm(args):

    output_prefix =             args['p']
    gtdb_gnm_metadata =         args['meta']
    gtdb_ref_gnm_path_txt =     args['path']
    taxons_to_retrieve_txt =    args['taxon']
    cpl_cutoff =                args['cpl']
    ctm_cutoff =                args['ctm']

    try:
        cpl_cutoff = int(cpl_cutoff)
    except:
        cpl_cutoff = cpl_cutoff

    try:
        ctm_cutoff = int(ctm_cutoff)
    except:
        ctm_cutoff = ctm_cutoff

    op_name_cpl_part = ''
    if cpl_cutoff is not None:
        op_name_cpl_part = '_%s' % cpl_cutoff

    op_name_ctm_part = ''
    if ctm_cutoff is not None:
        op_name_ctm_part = '_%s' % ctm_cutoff

    # define output file name
    gnms_to_retrieve_gtdb_txt = '%s%s%s_in_GTDB.txt'    % (output_prefix, op_name_cpl_part, op_name_ctm_part)
    gnms_to_retrieve_ncbi_txt = '%s%s%s_in_NCBI.txt'    % (output_prefix, op_name_cpl_part, op_name_ctm_part)
    gnms_taxonomy_txt         = '%s%s%s_taxonomy.txt'   % (output_prefix, op_name_cpl_part, op_name_ctm_part)

    already_exist_files = []
    for each_op in [gnms_to_retrieve_gtdb_txt, gnms_to_retrieve_ncbi_txt, gnms_taxonomy_txt]:
        if os.path.isfile(each_op) is True:
            already_exist_files.append(each_op)

    if len(already_exist_files) > 0:
        print('The following files already exist, use a different prefix with -p')
        print('\n'.join(already_exist_files))
        print('Program exited!')
        exit()

    # read in taxons_to_retrieve_txt
    taxons_to_retrieve_set = set()
    for each_taxon in open(taxons_to_retrieve_txt):
        taxons_to_retrieve_set.add(each_taxon.strip())

    # read in gtdb_ref_gnm_path_txt
    gtdb_ref_gnm_set = set()
    for each_gnm_path in open(gtdb_ref_gnm_path_txt):
        gnm_path_split = each_gnm_path.strip().split(' ')
        gnm_accession = gnm_path_split[0].split('_genomic')[0]
        gtdb_ref_gnm_set.add(gnm_accession)

    # get genomes from interested taxons
    gnm_cpl_dict = {}
    gnm_ctm_dict = {}
    gnm_size_dict = {}
    gnm_set_to_retrieve = set()
    gnm_to_taxon_dict = {}
    col_index = {}
    for each_ref in open(gtdb_gnm_metadata):
        each_ref_split = each_ref.strip().split('\t')
        if each_ref.startswith('accession	ambiguous_bases'):
            col_index = {key: i for i, key in enumerate(each_ref_split)}
        else:
            ref_accession = each_ref_split[0][3:]
            gnm_completeness  = float(each_ref_split[2])
            gnm_contamination = float(each_ref_split[3])
            gnm_size = each_ref_split[col_index['genome_size']]
            gtdb_taxonomy = each_ref_split[col_index['gtdb_taxonomy']]
            for each_interested_taxon in taxons_to_retrieve_set:
                with_semicolon = '%s;' % each_interested_taxon
                if with_semicolon in gtdb_taxonomy:

                    # check completeness
                    cpl_passed = False
                    if cpl_cutoff is None:
                        cpl_passed = True
                    else:
                        if gnm_completeness >= float(cpl_cutoff):
                            cpl_passed = True

                    # check contamination
                    ctm_passed = False
                    if ctm_cutoff is None:
                        ctm_passed = True
                    else:
                        if gnm_contamination <= float(ctm_cutoff):
                            ctm_passed = True

                    if (cpl_passed is True) and (ctm_passed is True):
                        gnm_set_to_retrieve.add(ref_accession)
                        gnm_to_taxon_dict[ref_accession] = gtdb_taxonomy
                        gnm_cpl_dict[ref_accession] = gnm_completeness
                        gnm_ctm_dict[ref_accession] = gnm_contamination
                        gnm_size_dict[ref_accession] = gnm_size

    # write out
    from_gtdb = 0
    from_ncbi = 0
    gnms_taxonomy_txt_handle = open(gnms_taxonomy_txt, 'w')
    gnms_taxonomy_txt_handle.write('Genome\tCompleteness\tContamination\tSize\tTaxon\n')
    gnms_to_retrieve_gtdb_txt_handle = open(gnms_to_retrieve_gtdb_txt, 'w')
    gnms_to_retrieve_ncbi_txt_handle = open(gnms_to_retrieve_ncbi_txt, 'w')
    for each_gnm_to_retrieve in sorted([i for i in gnm_set_to_retrieve]):
        gnm_taxon = gnm_to_taxon_dict.get(each_gnm_to_retrieve, 'NA')
        gnms_taxonomy_txt_handle.write('%s\t%s\t%s\t%s\t%s\n' % (each_gnm_to_retrieve, gnm_cpl_dict[each_gnm_to_retrieve], gnm_ctm_dict[each_gnm_to_retrieve], gnm_size_dict[each_gnm_to_retrieve], gnm_taxon))
        if each_gnm_to_retrieve in gtdb_ref_gnm_set:
            gnms_to_retrieve_gtdb_txt_handle.write('%s\n' % each_gnm_to_retrieve)
            from_gtdb += 1
        else:
            gnms_to_retrieve_ncbi_txt_handle.write('%s\n' % each_gnm_to_retrieve)
            from_ncbi += 1
    gnms_taxonomy_txt_handle.close()
    gnms_to_retrieve_gtdb_txt_handle.close()
    gnms_to_retrieve_ncbi_txt_handle.close()

    gnms_to_retrieve_gtdb_txt_with_num = '%s%s%s_in_GTDB_%s.txt'        % (output_prefix, op_name_cpl_part, op_name_ctm_part, from_gtdb)
    gnms_to_retrieve_ncbi_txt_with_num = '%s%s%s_in_NCBI_%s.txt'        % (output_prefix, op_name_cpl_part, op_name_ctm_part, from_ncbi)

    os.system('mv %s %s' % (gnms_to_retrieve_gtdb_txt, gnms_to_retrieve_gtdb_txt_with_num))
    os.system('mv %s %s' % (gnms_to_retrieve_ncbi_txt, gnms_to_retrieve_ncbi_txt_with_num))

    print('Total number of genomes from interested taxons: %s'  % len(gnm_set_to_retrieve))
    print('Genomes in GTDB: %s, see details in %s'              % (from_gtdb, gnms_to_retrieve_gtdb_txt_with_num))
    print('Genomes in NCBI: %s, see details in %s'              % (from_ncbi, gnms_to_retrieve_ncbi_txt_with_num))
    print('Genome taxon information is in %s'                   % (gnms_taxonomy_txt))


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-p',           required=False, default='Genomes',          help='output prefix')
    parser.add_argument('-meta',        required=True,                              help='GTDB reference genome metadata')
    parser.add_argument('-path',        required=True,                              help='GTDB genome_paths.tsv')
    parser.add_argument('-taxon',       required=True,                              help='interested taxons, one taxon per line')
    parser.add_argument('-cpl',         required=False, default=None, type=float,   help='completeness cutoff (0-100), default: None')
    parser.add_argument('-ctm',         required=False, default=None, type=float,   help='contamination cutoff, default: None')
    args = vars(parser.parse_args())
    get_GTDB_taxon_gnm(args)



import argparse
import numpy as np
from scipy.stats import mannwhitneyu


mannwhitneyu_usage = '''
============== mannwhitneyu example commands ============== 

BioSAK mannwhitneyu -i genome_size_by_lifestyle.txt

# file format: no header, tab separated.
1.61	nonsponge
1.59	nonsponge
1.47	sponge
1.52	sponge

===========================================================
'''


def mann_whitney_u(args):

    file_in = args['i']

    value_dict = dict()
    for each_s in open(file_in):
        each_s_split = each_s.strip().split('\t')
        s_cate  = each_s_split[1]
        s_value = float(each_s_split[0])
        if s_cate not in value_dict:
            value_dict[s_cate] = [s_value]
        else:
            value_dict[s_cate].append(s_value)

    if len(value_dict) != 2:
        cate_list = sorted(list(value_dict.keys()))
        print('Group number has to be 2, program exited!')
        print('Currently has: %s' % ','.join(cate_list))
        exit()

    cate_list_sorted = sorted(list(value_dict.keys()))
    value_lol = [value_dict[i] for i in cate_list_sorted]
    _, p_value = mannwhitneyu(value_lol[0], value_lol[1])

    print('sample\tsize\tmean\tstd')
    print('%st\t%s\t%s\t%s' % (cate_list_sorted[0], len(value_lol[0]), float("{0:.2f}".format(np.mean(value_lol[0]))), float("{0:.2f}".format(np.std(value_lol[0])))))
    print('%st\t%s\t%s\t%s' % (cate_list_sorted[1], len(value_lol[1]), float("{0:.2f}".format(np.mean(value_lol[1]))), float("{0:.2f}".format(np.std(value_lol[1])))))
    print('p-value: %s' % p_value)


if __name__ == '__main__':

    mannwhitneyu_parser = argparse.ArgumentParser()
    mannwhitneyu_parser.add_argument('-i', required=True, help='input file')
    args = vars(mannwhitneyu_parser.parse_args())
    mann_whitney_u(args)



import os
import argparse
import subprocess


metaBiosample_usage = '''
=========== metaBiosample example commands ===========

Dependency: efetch

BioSAK metaBiosample -f -i SAMEA5126984 -o op_dir
BioSAK metaBiosample -f -i biosamples.txt -o op_dir

=====================================================
'''

def check_executables(program_list):

    not_detected_programs = []
    for needed_program in program_list:
        if subprocess.call(['which', needed_program], stdout=open(os.devnull, 'wb')) != 0:
            not_detected_programs.append(needed_program)

    if not_detected_programs != []:
        print('%s not detected, program exited!' % ','.join(not_detected_programs))
        exit()


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


def efetch_op_to_dict(efetch_op_txt):

    metadata_dict = dict()
    for each_line in open(efetch_op_txt):
        each_line = each_line.strip()
        if not each_line.endswith(':'):
            if each_line.startswith('/'):
                each_line = each_line[1:]
                each_line_split = each_line.split('=')
                attribute_name = each_line_split[0]
                attribute_value = each_line_split[1][1:-1]
                metadata_dict[attribute_name] = attribute_value
            elif each_line.startswith('Identifiers: '):
                each_line = each_line[len('Identifiers: '):]
                each_line_split = each_line.split(';')
                for each_identifier in each_line_split:
                    each_identifier = each_identifier.strip()
                    each_identifier_split = each_identifier.split(': ')
                    metadata_dict[each_identifier_split[0]] = each_identifier_split[1]
            elif each_line.startswith('1: '):
                desc_line = each_line[len('1: '):]
                metadata_dict['sample_description'] = desc_line
            elif each_line.startswith('Organism: '):
                desc_line = each_line[len('Organism: '):]
                metadata_dict['organism'] = desc_line
            else:
                pass

    return metadata_dict


def metaBiosample(args):

    file_in         = args['i']
    op_dir          = args['o']
    force_overwrite = args['f']

    check_executables(['efetch'])

    tmp_dir               = '%s/tmp'            % op_dir
    combined_metadata_txt = '%s/metadata.txt'   % op_dir

    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('specified output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % tmp_dir)

    accession_set = set()
    if os.path.isfile(file_in) is True:
        for each_id in open(file_in):
            accession_set.add(each_id.strip())
    else:
        accession_set.add(file_in)

    efetch_cmd_list = []
    op_file_set = set()
    for each_id in accession_set:
        biosample_id = each_id.strip()
        op_meta_txt = '%s/%s.txt' % (tmp_dir, biosample_id)
        efetch_cmd = 'efetch -db biosample -id %s -format abstract > %s' % (biosample_id, op_meta_txt)
        efetch_cmd_list.append(efetch_cmd)
        op_file_set.add(op_meta_txt)

    cmd_txt = '%s/cmd.txt' % op_dir
    with open(cmd_txt, 'w') as cmd_txt_handle:
        cmd_txt_handle.write('\n'.join(sorted(efetch_cmd_list)) + '\n')

    for each_cmd in efetch_cmd_list:
        os.system(each_cmd)

    all_attr_set = set()
    metadata_dod = dict()
    for each_file in op_file_set:
        f_name, f_path, f_base, f_ext = sep_path_basename_ext(each_file)
        current_metadata_dict = efetch_op_to_dict(each_file)
        for i in current_metadata_dict:
            all_attr_set.add(i)
        metadata_dod[f_base] = current_metadata_dict

    all_attr_list_sorted = sorted(list(all_attr_set))

    combined_metadata_txt_handle = open(combined_metadata_txt, 'w')
    combined_metadata_txt_handle.write('Biosample\t%s\n' % ('\t'.join(all_attr_list_sorted)))
    for each_biosample in sorted(list(metadata_dod.keys())):
        current_biosample_attr_list = [each_biosample]
        for each_attr in all_attr_list_sorted:
            current_biosample_attr_list.append(metadata_dod[each_biosample].get(each_attr, 'na'))
        combined_metadata_txt_handle.write('\t'.join(current_biosample_attr_list) + '\n')
    combined_metadata_txt_handle.close()


if __name__ == '__main__':

    arg_parser = argparse.ArgumentParser(usage=metaBiosample_usage)
    arg_parser.add_argument('-i',   required=True,                        help='biosample id file, one id per line')
    arg_parser.add_argument('-o',   required=True,                        help='output folder')
    arg_parser.add_argument('-f',   required=False, action="store_true",  help='Force overwrite existing results')
    args = vars(arg_parser.parse_args())
    metaBiosample(args)



import os
import argparse
import pandas as pd


subset_df_usage = '''
============================ subset_df example commands ============================

BioSAK subset_df -i demo_df.txt -r row_id.txt -o df_subset.txt
BioSAK subset_df -i demo_df.txt -c col_id.txt -o df_subset.txt -e
BioSAK subset_df -i demo_df.txt -r row_id.txt -c col_id.txt -o df_subset.txt -e
BioSAK subset_df -i demo_df.txt -r row_id.txt -c col_id.txt -o df_subset.txt -b -m

# Note
1. If the "-c" or "-r" file contains multiple columns (tab separated), the 1st 
   column will be used. 
2. If you want to skip the 1st row in the "-c" or "-r" file, provide "-skip1row".

====================================================================================
'''


def subset_df(args):

    file_in             = args['i']
    file_out            = args['o']
    rows_to_keep_file   = args['r']
    cols_to_keep_file   = args['c']
    df_separator        = args['s']
    in_binary           = args['b']
    zero_as_minus_one   = args['m']
    skip1row            = args['skip1row']
    exclude_rows_cols   = args['e']
    column_name_pos     = 0
    row_name_pos        = 0

    # setup separator
    if df_separator in ['tab', 'Tab', 'TAB']:
        sep_symbol = '\t'
    elif df_separator in ['comma', 'Comma', 'COMMA']:
        sep_symbol = ','
    else:
        print('Please specify separator as either tab or comma, program exited!')
        exit()

    ###################################### get the id of rows and cols to subset #######################################

    # put all row and col headers in list
    df = pd.read_csv(file_in, sep=sep_symbol, header=column_name_pos, index_col=row_name_pos)
    df_row_header_list = df.index.values.tolist()
    df_col_header_list = df.columns.values.tolist()

    # read in rows_to_keep_file
    rows_found_set = set()
    rows_missing_set = set()
    if rows_to_keep_file is not None:
        if os.path.isfile(rows_to_keep_file) is False:
            print('%s not found, program exited!' % rows_to_keep_file)
            exit()
        else:
            file_row_index = 0
            for each_r in open(rows_to_keep_file):
                ignore_current_row = False
                if (skip1row is True) and (file_row_index == 0):
                    ignore_current_row = True
                if ignore_current_row is False:
                    row_id = each_r.strip().split()[0]
                    if row_id in df_row_header_list:
                        rows_found_set.add(row_id)
                    else:
                        rows_missing_set.add(row_id)
                file_row_index += 1

    # read in cols_to_keep_file
    cols_found_set = set()
    cols_missing_set = set()
    if cols_to_keep_file is not None:
        if os.path.isfile(cols_to_keep_file) is False:
            print('%s not found, program exited!' % cols_to_keep_file)
            exit()
        else:
            file_row_index = 0
            for each_c in open(cols_to_keep_file):
                ignore_current_row = False
                if (skip1row is True) and (file_row_index == 0):
                    ignore_current_row = True
                if ignore_current_row is False:
                    col_id = each_c.strip().split()[0]
                    if col_id in df_col_header_list:
                        cols_found_set.add(col_id)
                    else:
                        cols_missing_set.add(col_id)
                file_row_index += 1

    # report
    if len(rows_missing_set) > 0:
        print('The following rows are missing from the dataframe:\n%s'    % ','.join(sorted(list(rows_missing_set))))

    if len(cols_missing_set) > 0:
        print('The following columns are missing from the dataframe:\n%s' % ','.join(sorted(list(cols_missing_set))))

    ####################################################################################################################

    rows_to_keep_set = set()
    cols_to_keep_set = set()
    if exclude_rows_cols is False:
        rows_to_keep_set = rows_found_set
        cols_to_keep_set = cols_found_set
    else:
        for each_row in df_row_header_list:
            if each_row not in rows_found_set:
                rows_to_keep_set.add(each_row)
        for each_col in df_col_header_list:
            if each_col not in cols_found_set:
                cols_to_keep_set.add(each_col)

    # turn sets into lists
    rows_to_keep_list_sorted = sorted(list(rows_to_keep_set))
    cols_to_keep_list_sorted = sorted(list(cols_to_keep_set))

    if len(rows_to_keep_list_sorted) == 0:
        if len(cols_to_keep_list_sorted) == 0:
            subset_df = df.loc[:, :]
        else:
            subset_df = df.loc[:, cols_to_keep_list_sorted]
    else:
        if len(cols_to_keep_list_sorted) == 0:
            subset_df = df.loc[rows_to_keep_list_sorted, :]
        else:
            subset_df = df.loc[rows_to_keep_list_sorted, cols_to_keep_list_sorted]

    if in_binary is True:
        subset_df[subset_df <= 0] = 0
        subset_df[subset_df > 0] = 1

    # turn 0 to -1
    if zero_as_minus_one is True:
        subset_df[subset_df == 0] = -1

    subset_df.to_csv(file_out, sep=sep_symbol)


if __name__ == '__main__':

    subset_df_parser = argparse.ArgumentParser(usage=subset_df_usage)
    subset_df_parser.add_argument('-i',         required=True,                          help='input file')
    subset_df_parser.add_argument('-o',         required=True,                          help='output file')
    subset_df_parser.add_argument('-c',         required=False, default=None,           help='header of columns to keep')
    subset_df_parser.add_argument('-r',         required=False, default=None,           help='header of rows to keep')
    subset_df_parser.add_argument('-s',         required=False, default='tab',          help='column separator, choose from tab and comma, default: tab')
    subset_df_parser.add_argument('-b',         required=False, action='store_true',    help='write out dataframe in 0/1 format')
    subset_df_parser.add_argument('-e',         required=False, action='store_true',    help='subset df by excluding the specified rows/columns')
    subset_df_parser.add_argument('-m',         required=False, action='store_true',    help='convert 0 to -1')
    subset_df_parser.add_argument('-skip1row',  required=False, action='store_true',    help='skip the 1st row of the -c/-r file')
    args = vars(subset_df_parser.parse_args())
    subset_df(args)



import os
import glob
import argparse


Tax4Fun2IndOTU_usage = '''
============================== Tax4Fun2IndOTU example commands ==============================

=============================================================================================
'''


def Tax4Fun2IndOTU(args):

    print(args)


if __name__ == '__main__':

    COG_parser = argparse.ArgumentParser(usage=Tax4Fun2IndOTU_usage)
    COG_parser.add_argument('-i',               required=True,                              help='path to input sequences (in multi-fasta format)')
    COG_parser.add_argument('-x',               required=False,                             help='file extension')
    COG_parser.add_argument('-m',               required=True,                              help='sequence type, "N/n" for "nucleotide", "P/p" for "protein"')
    COG_parser.add_argument('-depth',           required=False, default=None,               help='gene depth file/folder')
    COG_parser.add_argument('-pct_by_all',      required=False, action='store_true',        help='normalize by all query genes, including those without COG assignment')
    COG_parser.add_argument('-db_dir',          required=True,                              help='DB folder')
    COG_parser.add_argument('-diamond',         required=False, action='store_true',        help='run diamond (for big dataset), default is NCBI blastp')
    COG_parser.add_argument('-t',               required=False, type=int, default=1,        help='number of threads')
    COG_parser.add_argument('-evalue',          required=False, default=0.001, type=float,  help='evalue cutoff, default: 0.001')
    args = vars(COG_parser.parse_args())
    Tax4Fun2IndOTU(args)



import os
import argparse


subset_GTDB_meta_usage = '''
================================ subset_GTDB_meta example commands ================================

BioSAK subset_GTDB_meta -meta ar122_metadata_r202.tsv -id gnm_id.txt -out metadata_subset.txt
BioSAK subset_GTDB_meta -meta bac120_metadata_r202.tsv -id gnm_id.txt -out metadata_subset.txt

# format of gnm_id.txt (one id per line)
GCA_002726655.1
GCA_002728675.1

===================================================================================================
'''


def subset_GTDB_meta(args):

    gnm_metadata   = args['meta']
    interested_gnm = args['id']
    output_txt     = args['out']

    interested_gnm_set = set()
    for each_gnm in open(interested_gnm):
        interested_gnm_set.add(each_gnm.strip())

    found_gnm_set = set()
    output_txt_handle = open(output_txt, 'w')
    for each_ref in open(gnm_metadata):
        each_ref_split = each_ref.strip().split('\t')
        if each_ref.startswith('accession	ambiguous_bases'):
            output_txt_handle.write(each_ref)
        else:
            ref_accession = each_ref_split[0][3:]
            if ref_accession in interested_gnm_set:
                output_txt_handle.write(each_ref)
                found_gnm_set.add(ref_accession)
    output_txt_handle.close()

    if len(found_gnm_set) < len(interested_gnm_set):
        not_found_gnm_list = []
        for each_gnm in interested_gnm_set:
            if each_gnm not in found_gnm_set:
                not_found_gnm_list.append(each_gnm)
        print('The following genomes were not found:')
        print(','.join(not_found_gnm_list))


if __name__ == '__main__':

    parser = argparse.ArgumentParser(usage=subset_GTDB_meta_usage)
    parser.add_argument('-meta', required=True, help='GTDB reference genome metadata')
    parser.add_argument('-id',   required=True, help='id of genomes')
    parser.add_argument('-out',  required=True, help='output file')
    args = vars(parser.parse_args())
    subset_GTDB_meta(args)



#!/usr/bin/env python

__author__ = "Xiang Gao, Huaiying Lin, Qunfeng Dong"
__copyright__ = "Copyright 2016, Bayesian-based LCA taxonomic classification"
__credits__ = ["Xiang Gao", "Huaiying Lin", "Qunfeng Dong"]
__license__ = "GPL"
__version__ = "2.1"
__maintainer__ = "Huaiying Lin"
__email__ = "hlin2@luc.edu"
__status__ = "Python3"

import os
from sys import exit, stderr
from collections import defaultdict

try:
    from Bio import AlignIO, SeqIO
except ImportError:
    stderr.write("Error! BioPython is not detected!\n")
    exit(1)


import time
import random
import argparse

startTime = time.time()

def float_1range(x):
    x = float(x)
    if x < 0.0 or x > 1.0:
        raise argparse.ArgumentTypeError("%r not in range [0.0, 1.0]" % (x,))
    return x

def float_100range(x):
    x = float(x)
    if x < 0.0 or x > 100.0:
        raise argparse.ArgumentTypeError("%r not in range [0.0, 100.0]" % (x,))
    return x


def check_taxdb():
    ''' Check whether BLASTDB environmental variable has been setup'''
    if 'BLASTDB' not in os.environ.keys():
        print(
            "ERROR: taxdb location has not been set up as the environmental variable BLASTDB. Please set up as \n\texport BLASTDB=/location/of/taxdb.bti/and/taxdb.btd/")
        exit(1)


def check_program(prgname):
    '''Check whether a program has been installed and put in the PATH'''
    import os
    path = os.popen("which " + prgname).read().rstrip()
    if len(path) > 0 and os.path.exists(path):
        # print(prgname + " is located in your PATH!")
        pass
    else:
        print("ERROR: " + prgname + " is NOT in your PATH, please set up " + prgname + "!")
        exit(1)


def run_blastn(fsa, eset, nsub, db, proc, op_dir, seq_file_base):
    '''Running blastn with customized output format'''
    check_program("blastn")
    import subprocess
    # print("> > Running blast!!")
    # subprocess.call(['blastn','-query',fsa,'-evalue',str(eset),'-dust','no','-soft_masking','false','-db',db,'-num_threads',str(proc),'-outfmt','6 std score sstrand slen qlen','-max_target_seqs',str(nsub),'-out',fsa+'.blastn'])
    subprocess.call(
        ['blastn', '-query', fsa, '-evalue', str(eset), '-perc_identity', '90', '-soft_masking', 'false', '-db', db,
         '-num_threads', str(proc), '-outfmt', '6 std score sstrand slen qlen', '-max_target_seqs', str(nsub), '-out',
         op_dir + '/' + seq_file_base + '.blastn'])
    # print("> > Blastn Finished!!")


def get_dic_from_aln(aln):
    '''Read in alignment and convert it into a dictionary'''
    alignment = AlignIO.read(aln, "clustal")
    alndic = {}
    for r in alignment:
        alndic[r.id] = list(r.seq)
    return alndic


def pairwise_score(alnmtdic, query, match, mismatch, ngap):
    '''Calculate pairwise alignment score given a query'''
    nt = ["A", "C", "T", "G", "g", "a", "c", "t"]
    hitscore = defaultdict(float)
    for k, v in alnmtdic.items():
        if k != query:
            for i in range(len(v)):
                if (alnmtdic[query][i] in nt) and (v[i] in nt) and (alnmtdic[query][i] == v[i]):
                    hitscore[k] += match
                elif (alnmtdic[query][i] not in nt) and (v[i] not in nt) and (alnmtdic[query][i] == v[i]):
                    hitscore[k] += 0
                elif ((alnmtdic[query][i] not in nt) or (v[i] not in nt)) and (alnmtdic[query][i] != v[i]):
                    hitscore[k] += mismatch
                elif (alnmtdic[query][i] in nt) and (v[i] in nt) and (alnmtdic[query][i] != v[i]):
                    hitscore[k] += ngap
    total = float(sum(hitscore.values()))
    if total <= 0:
        total = 1
    for k, v in hitscore.items():
        hitscore[k] = v / total
    return hitscore


def random_aln_score(alnmtdic, query, match, mismatch, ngap):
    '''Randomize the alignment, and calculate the score'''
    nt = ["A", "C", 'T', "G", "g", "a", "c", "t"]
    idx = []
    alnlist = list(alnmtdic.values())
    for i in range(len(alnlist[0])):  # 'dict_values' object does not support indexing in python 3
        idx.append(random.choice(range(len(alnlist[0]))))
    hitscore = defaultdict(float)
    for k, v in alnmtdic.items():
        if k != query:
            for i in idx:
                if (alnmtdic[query][i] in nt) and (v[i] in nt) and (alnmtdic[query][i] == v[i]):
                    hitscore[k] += match
                elif (alnmtdic[query][i] not in nt) and (v[i] not in nt) and (alnmtdic[query][i] == v[i]):
                    hitscore[k] += 0
                elif ((alnmtdic[query][i] not in nt) or (v[i] not in nt)) and (alnmtdic[query][i] != v[i]):
                    hitscore[k] += mismatch
                elif (alnmtdic[query][i] in nt) and (v[i] in nt) and (alnmtdic[query][i] != v[i]):
                    hitscore[k] += ngap
    return hitscore


def get_gap_pos(query, alnmtdic):
    '''Get the gap position in the alignment'''
    for i in range(len(alnmtdic[query])):
        if alnmtdic[query][i] != "-":
            start = i
            break
    for i in range(len(alnmtdic[query]) - 1, 0, -1):
        if alnmtdic[query][i] != "-":
            end = i
            break
    return start, end


def cut_gap(alndic, start, end):
    '''Given a start and end gap position, truncate the alignmnet'''
    trunc_alndic = {}
    for k_truc, v_truc in alndic.items():
        trunc_alndic[k_truc] = v_truc[start:end]
    return trunc_alndic


def read_tax_acc(taxfile, IDlen):
    acctax = {}
    # print(">  > Reading in taxonomy information! ....")
    with open(taxfile) as tx:
        for l in tx:
            lne = l.rstrip().strip(";").split("\t")
            if len(lne) == 2:
                if len(lne[0]) > IDlen:
                    print("ERROR: Your reference sequence ID length longer than 32, please shorten your ID length and try again!! ")
                    exit(1)
                else:
                    acctax[lne[0].split(".")[0]] = dict(x.split(":", 1) for x in lne[1].split(";"))
    if len(acctax) == 0:
        print("ERROR: No taxonmy record was read in. Please examine the format of your taxonomy file according to the instruction on github.")
        exit(1)
    return acctax


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]

##### parser ######

parser = argparse.ArgumentParser(description=
                                 ''' << Bayesian-based LCA taxonomic classification method >>\n\n   Please make sure the following softwares are in your PATH:
    1.muscle (http://www.drive5.com/muscle/downloads.htm), muscle should be the program's name.
    2.ncbi-blast suite (ftp://ftp.ncbi.nlm.nih.gov/blast/executables/blast+/LATEST/)
    3.clustalo (http://www.clustal.org/omega/), clustalo should be the program's name.
    4.Biopython should be installed locally.''', epilog="No warrenty comes with this script. Author: hlin2@luc.edu. \nAny suggestions or bugs report are welcomed.", add_help=False, formatter_class=argparse.RawTextHelpFormatter)

##### Required arguments #####
required = parser.add_argument_group('required arguments')
required.add_argument("-i", "--fsa", help="Input fasta file", type=str, required=True)

##### Taxonomy filtering arguments #####
taxoptions = parser.add_argument_group('taxonomy profiling options [filtering of hits]')
taxoptions.add_argument("-x", "--skipblast",    help="skip blastn. Default: blastn is not skipped", action='store_true')
taxoptions.add_argument("-n", "--nper",         help="number of times to bootstrap. Default: 100", type=int, default=100)
taxoptions.add_argument("-j", "--nsub",         help="maximum number of subjects to include for each query reads. Default: 50", type=int, default=50)
taxoptions.add_argument("-d", "--topper",       help="proportion of hits to include from top hit. Default: 0.1 [0-1]",    type=float_1range, default=0.1)
taxoptions.add_argument("-e", "--eset",         help="minimum evalue to include for blastn. Default: 0.1", type=float,  default=0.1)
taxoptions.add_argument("-b", "--bset",         help="minimum bitscore to include for blastn hits. Default: 100", type=int, default=100)
taxoptions.add_argument("-c", "--cvrset",       help="minimum coverage to include. Default: 0.85 [0-1]", type=float_1range,   default=0.85)
taxoptions.add_argument("--iset",               help="minimum identity score to include. Default: 90 [0-100]", type=float_100range,   default=90.0)

##### Alignment control arguments #####
alignoptions = parser.add_argument_group('alignment control arguments')
alignoptions.add_argument("-a", "--align", default='clustalo',      help="alignment tool: clustal omega or muscle. Default: clustalo", type=str,  choices=["clustalo", "muscle"])
alignoptions.add_argument("-m", "--match", default=1.0,             help="alignment match score. Default: 1", type=float)
alignoptions.add_argument("-f", "--mismatch", default=-2.5,         help="alignment mismatch penalty. Default: -2.5",   type=float)
alignoptions.add_argument("-g", "--ngap", default=-2.0,             help="alignment gap penalty. Default: -2", type=float)

##### Other arguments #####
optional = parser.add_argument_group('other arguments')
optional.add_argument("-r", "--tax", default='./db/16S_ribosomal_RNA.ACC.taxonomy', help="reference taxonomy file for the Database. Default: db/16S_ribosomal_RNA.taxonomy", type=str)
optional.add_argument("-q", "--db", default='./db/16S_ribosomal_RNA',               help="refernece blast database. Default: db/16S_ribosomal_RNA", type=str)
optional.add_argument("-t", "--gap", default=10,                                    help="extra number of nucleotides to include at the beginning and end of the hits. Default: 10",    type=int)
optional.add_argument("-o", "--outdir",                                             help="output directory", type=str)
optional.add_argument("-p", "--proc", default=2,                                    help="how many processors are used in blastn step. Default: 2 processors", type=int)
optional.add_argument("-h", "--help",                                               help="show this help message and exit", action="help")

##### parse arguments #####
args        = parser.parse_args()
out_dir     = args.outdir
input_fa    = args.fsa

fa_name, fa_path, fa_base, fa_ext = sep_path_basename_ext(input_fa)

##### Output file name ####

op_txt = '%s/%s.blca.out'    % (out_dir, fa_base)

################################################################

## check taxdb
## check whether blastdbcmd is located in the path
##check_program("blastdbcmd")

## check whether alignment tool is located in the path
if args.align == "clustalo":
    check_program("clustalo")
    IDlenallow = 100
elif args.align == "muscle":
    check_program("muscle")
    IDlenallow = 32

### read in input fasta file ###
fsadic = {}
with open(args.fsa) as f:
    for r in SeqIO.parse(f, "fasta"):
        if len(r.id) > IDlenallow:
            print("ERROR: Your query ID length is longer than " + str(
                IDlenallow) + ", please shorten your ID length and try again!!")
            exit(1)
        else:
            fsadic[r.id] = str(r.seq)
# print(">  > Fasta file read in!")

### read in pre-formatted lineage information ###
acc2tax = read_tax_acc(args.tax, IDlen=IDlenallow)

## Run blastn and output fas.blastn output
if not args.skipblast:
    run_blastn(args.fsa, args.eset, args.nsub, args.db, args.proc, out_dir, fa_base)
elif args.skipblast:
    print(">  > blastn is skipped")

## read in blastn output
# print(">  > read in blast file...")
b = open(out_dir + '/' + fa_base + '.blastn', 'r')
# print(">  > blastn file opened")
qtosdic = {}
giinfo = {}
## place holder for tracking input reads
sublist = set()  # major speed boost when list replaced by set
for ln in b:
    ln = ln.rstrip()
    line = ln.split("\t")
    sstart = int(line[8])
    send = int(line[9])
    sstrand = line[13]
    slen = int(line[14])
    bltscore = float(line[12])
    evalue = float(line[10])
    identity = float(line[2])
    ### extend sstart and send by gap ###
    if (sstart - args.gap) < 1:
        sstart = 1
    else:
        sstart -= args.gap
    if (send + args.gap) > slen:
        send = slen
    else:
        send += args.gap
    ### calculate coverage ###
    coverage = float(int(line[7]) - int(line[6]) + 1) / float(line[15])
    ### filter hits ###
    if evalue <= args.eset and identity >= args.iset and coverage >= args.cvrset and float(line[11]) >= args.bset:  # minimum values to include for blastn hits can be equal to user defined values
        giinfo[line[1] + ":" + str(sstart) + "-" + str(send)] = [sstart, send, sstrand, bltscore, evalue, identity]
        if line[0].replace("|", "_") not in sublist:
            subcount = 1
            topsc = float(line[11])
            qtosdic[line[0].replace("|", "_")] = [line[1] + ":" + str(sstart) + "-" + str(send)]
            sublist.add(line[0].replace("|", "_"))
        else:
            bitsc = float(line[11])
            if (topsc - bitsc) / topsc < args.topper and subcount <= args.nsub:
                qtosdic[line[0].replace("|", "_")].append(line[1] + ":" + str(sstart) + "-" + str(send))
                subcount += 1
b.close()
# print(">  > blast output read in")

# print(">  > Start aligning reads...")
# print("Cut offs")
# print("evalue:",args.eset,"identity:", args.iset, "coverage:", args.cvrset)
# print qtosdic.values()
os.system("rm -f " + op_txt)
outout = open(op_txt, 'w+')
levels = ["superkingdom", "phylum", "class", "order", "family", "genus", "species"]
for seqn in fsadic.keys():
    # for k1,v1 in qtosdic.items():
    k1 = seqn
    if k1 in qtosdic and k1 not in acc2tax:
        v1 = qtosdic[k1]
        os.system("rm -f " + k1 + ".dblist")
        os.system("rm -f " + k1 + ".hitdb.fsa")
        ### Get all the hits list belong to the same query ###
        msafsa = open(out_dir + '/' + k1 + ".dblist", 'w')
        for g in v1:
            # if blastn returns hits on minus strand, reverse sstart and send positions in -range arg
            if giinfo[g][1] < giinfo[g][0]:
                msafsa.write(g.split(":")[0] + " " + str(giinfo[g][1]) + "-" + str(giinfo[g][0]) + " " + giinfo[g][2] + "\n")
            else:
                msafsa.write(g.split(":")[0] + " " + str(giinfo[g][0]) + "-" + str(giinfo[g][1]) + " " + giinfo[g][2] + "\n")
        msafsa.close()
        os.system("blastdbcmd -db " + args.db + " -entry_batch " + out_dir + '/' + k1 + ".dblist -outfmt %f > " + out_dir + '/' + k1 + ".hitdb.fsa")
        ### Add query fasta sequence to extracted hit fasta ###
        fifsa = open(out_dir + '/' + k1 + ".hitdb.fsa", 'a')
        fifsa.write(">" + k1 + "\n" + fsadic[k1])
        fifsa.close()
        os.system("rm " + out_dir + '/' + k1 + ".dblist")
        ### Run alignment ###
        if args.align == "clustalo":
            os.system("clustalo --threads=" + str(args.proc) + " --outfmt=clu -i " + out_dir + '/' + k1 + ".hitdb.fsa -o " + out_dir + '/' + k1 + ".aln")
        if args.align == "muscle":
            os.system("muscle -quiet -clw -in " + k1 + ".hitdb.fsa -out " + k1 + ".aln")
        alndic = get_dic_from_aln(out_dir + '/' + k1 + ".aln")
        os.system("rm " + out_dir + '/' + k1 + ".hitdb.fsa")
        os.system("rm " + out_dir + '/' + k1 + ".aln")
        # print "Processing:",k1
        ### get gap position and truncate the alignment###
        start, end = get_gap_pos(k1, alndic)
        trunc_alndic = cut_gap(alndic, start, end)
        orgscore = pairwise_score(trunc_alndic, k1, args.match, args.mismatch, args.ngap)
        ### start bootstrap ###
        perdict = {}  # record alignmet score for each iteration
        pervote = {}  # record vote after nper bootstrap
        ### If any equal score, average the vote ###
        for j in range(args.nper):
            tmpdic = random_aln_score(trunc_alndic, k1, args.match, args.mismatch, args.ngap)
            # print("iter: ", j, ":", tmpdic) # for checking purpose
            perdict[j] = tmpdic
            mx = max(tmpdic.values())
            mxk = [k3 for k3, v3 in tmpdic.items() if v3 == mx]
            ### check if there are multiple max hits ###
            if len(mxk) == 1:
                if max(tmpdic, key=tmpdic.get) in pervote:
                    pervote[max(tmpdic, key=tmpdic.get)] += float(1)
                else:
                    pervote[max(tmpdic, key=tmpdic.get)] = float(1)
            else:
                por = float(1) / len(mxk)
                for h in mxk:
                    if h in pervote:
                        pervote[h] += por
                    else:
                        pervote[h] = por
        ### normalize vote by total votes ###
        ttlvote = sum(pervote.values())
        for k4, v4 in pervote.items():
            pervote[k4] = v4 / ttlvote * 100
        ### hit taxnomy parsing ###
        hitstax = {}
        for k5, v5 in orgscore.items():
            shortk5 = k5.split(":")[0].split(".")[0]
            if shortk5 in acc2tax:
                k2tax = acc2tax[shortk5]
                mistx = list(set(levels) - set(k2tax.keys()))
                for mis in mistx:
                    k2tax[mis] = "NA"
                if k5 not in pervote:
                    pervote.update({k5: 0})
                hitstax[k5] = [k2tax, pervote[k5], orgscore[k5]]
        voten = [y[1] for y in hitstax.values()]
        prbn = [z[2] for z in hitstax.values()]
        outout.write(k1 + "\t")
        for le in levels:
            lex = [x[0][le] for x in hitstax.values()]
            lexsum = {}
            prosum = {}
            for d in range(len(lex)):
                # print voten[d]
                if lex[d] in lexsum:
                    lexsum[lex[d]] += voten[d]
                    prosum[lex[d]] += prbn[d]
                else:
                    lexsum[lex[d]] = voten[d]
                    prosum[lex[d]] = prbn[d]
            outout.write(le + ":" + max(lexsum, key=lexsum.get) + ";" + str(max(lexsum.values())) + ";")
        outout.write("\n")
    elif k1 in acc2tax:
        print("[WARNING] Your sequence " + k1 + " has the same ID as the reference database! Please correct it!")
        print("...Skipping sequence " + k1 + " ......")
        outout.write(k1 + "\tSkipped\n")
    else:
        outout.write(k1 + "\tUnclassified\n")

outout.close()
# print(">  > Taxonomy file generated!!")

endTime = time.time()
# print("Time elapsed: %i minutes" % (round((endTime - startTime) / 60)))



import os
import glob
import argparse
from Bio import SeqIO
import multiprocessing as mp


gbk2fna_usage = '''
========= gbk2fna example commands =========

BioSAK gbk2fna -i MAG.gbk -o MAG.fna
BioSAK gbk2fna -i gbk_dir -o fna_dir -t 6

============================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def gbk2fna_single(arg_list):
    gbk_in  = arg_list[0]
    fna_out = arg_list[1]
    SeqIO.convert(gbk_in, 'genbank', fna_out, 'fasta')


def gbk2fna(args):

    gbk_in      = args['i']
    in_ext      = args['ix']
    fna_out     = args['o']
    out_ext     = args['ox']
    num_threads = args['t']

    if os.path.isfile(gbk_in) is True:
        gbk2fna_single([gbk_in, fna_out])

    elif os.path.isdir(gbk_in) is True:

        gbk_file_re   = '%s/*.%s' % (gbk_in, in_ext)
        gbk_file_list = glob.glob(gbk_file_re)

        if len(gbk_file_list) == 0:
            print('No input gbk found, program exited!')
            exit()

        if os.path.isdir(fna_out) is True:
            print('output folder detected, program exited!')
            exit()
        os.system('mkdir %s' % fna_out)

        mp_arg_lol = []
        for each_gbk in gbk_file_list:
            gbk_path, gbk_base, gbk_ext = sep_path_basename_ext(each_gbk)
            pwd_fna_out = '%s/%s.%s' % (fna_out, gbk_base, out_ext)
            #gbk2fna_single([each_gbk, pwd_fna_out])
            mp_arg_lol.append([each_gbk, pwd_fna_out])

        pool = mp.Pool(processes=num_threads)
        pool.map(gbk2fna_single, mp_arg_lol)
        pool.close()
        pool.join()

    else:
        print('No input gbk found, program exited!')
        exit()


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('-i',  required=True,                   help='input gbk')
    parser.add_argument('-ix', required=False, default='gbk',   help='input file extension, default: gbk')
    parser.add_argument('-o',  required=True,                   help='output fna (contig sequences)')
    parser.add_argument('-ox', required=False, default='fna',   help='output file extension, default: fna')
    parser.add_argument('-t',  required=False, type=int, default=1,        help='number of threads')
    args = vars(parser.parse_args())
    gbk2fna(args)



import os
import glob
import argparse
import subprocess
import multiprocessing as mp


metaAssembly_usage = '''
==================== metaAssembly example commands ====================

BioSAK metaAssembly -i GCA_947846245.1 -o output_dir -f
BioSAK metaAssembly -i assembly_id.txt -o output_dir -f -t 12

# format of assembly_id.txt (one id per line)
GCA_947846245.1
GCF_026914265.1
GCF_002022765.2

Dependencies: datasets and dataformat
https://www.ncbi.nlm.nih.gov/datasets/docs/v2/download-and-install/

=======================================================================
'''


def check_executables(program_list):

    not_detected_programs = []
    for needed_program in program_list:

        if subprocess.call(['which', needed_program], stdout=open(os.devnull, 'wb')) != 0:
            not_detected_programs.append(needed_program)

    if not_detected_programs != []:
        print('%s not detected, program exited!' % ','.join(not_detected_programs))
        exit()


def get_nonredundant_table(file_in, file_out, ignore_na):
    col_dict = dict()
    col_list = []
    col_index_dict = dict()
    line_num_index = 0
    for each_line in open(file_in):
        line_num_index += 1
        line_split = each_line.strip().split('\t')
        if line_num_index == 1:
            col_list = line_split
            col_index_dict = {key: i for i, key in enumerate(line_split)}
        else:
            for col in col_list:
                if col not in col_dict:
                    col_dict[col] = [line_split[col_index_dict[col]]]
                else:
                    col_dict[col].append(line_split[col_index_dict[col]])

    # write out
    file_out_handle = open(file_out, 'w')
    for col in col_list:

        value_list_uniq = list(set(col_dict[col]))
        if value_list_uniq == ['']:
            value_list_uniq = ['na']

        if col not in ['Assembly BioSample Attribute Name', 'Assembly BioSample Attribute Value']:
            if ignore_na is True:
                if value_list_uniq != ['na']:
                    file_out_handle.write('%s:\t%s\n' % (col, ','.join(value_list_uniq)))
            else:
                file_out_handle.write('%s:\t%s\n' % (col, ','.join(value_list_uniq)))
        else:
            biosample_attribute_name_list = col_dict.get('Assembly BioSample Attribute Name', [])
            biosample_attribute_value_list = col_dict.get('Assembly BioSample Attribute Value', [])
            for (name, value) in zip(biosample_attribute_name_list, biosample_attribute_value_list):
                file_out_handle.write('Assembly BioSample Attribute %s:\t%s\n' % (name, value))
    file_out_handle.close()


def summarise_metadata(file_dir, file_ext, summary_txt):

    file_re = '%s/*.%s' % (file_dir, file_ext)
    file_list = glob.glob(file_re)

    all_col_name_set = set()
    sample_metadata_dod = dict()
    for each_file in file_list:
        file_name = os.path.basename(each_file)
        sample_id = file_name[:-(len(file_ext) + 1)]
        current_sample_metadata_dict = dict()
        col_index = dict()
        line_num_index = 0
        for each_line in open(each_file):
            line_num_index += 1
            line_split = each_line.split('\t')
            if line_num_index == 1:
                col_index = {key: i for i, key in enumerate(line_split)}
            else:
                bioSample_attribute_name = line_split[col_index['Assembly BioSample Attribute Name']]
                bioSample_attribute_value = line_split[col_index['Assembly BioSample Attribute Value']]
                for each_col in col_index:
                    if each_col not in ['Assembly BioSample Attribute Name', 'Assembly BioSample Attribute Value']:
                        if each_col not in current_sample_metadata_dict:
                            current_sample_metadata_dict[each_col] = set()
                        current_sample_metadata_dict[each_col].add(line_split[col_index[each_col]].strip())
                        all_col_name_set.add(each_col.strip())
                    else:
                        if each_col == 'Assembly BioSample Attribute Name':
                            new_col_name = 'Assembly BioSample Attribute Name - %s' % bioSample_attribute_name
                            if bioSample_attribute_name not in current_sample_metadata_dict:
                                current_sample_metadata_dict[new_col_name] = set()
                            current_sample_metadata_dict[new_col_name].add(bioSample_attribute_value)
                            all_col_name_set.add(new_col_name.strip())
        sample_metadata_dod[sample_id] = current_sample_metadata_dict

    summary_txt_handle = open(summary_txt, 'w')
    summary_txt_handle.write('ID\t%s\n' % '\t'.join(sorted(list(all_col_name_set))))
    for each_sample in sample_metadata_dod:
        metadata_dict = sample_metadata_dod[each_sample]
        value_list = []
        value_list.append(each_sample)
        for each_col in sorted(list(all_col_name_set)):
            col_value = metadata_dict.get(each_col, set())
            col_value_str = ','.join(sorted(list(col_value)))
            if col_value_str == '':
                col_value_str = 'na'
            value_list.append(col_value_str)
        str_to_write = '\t'.join(value_list)
        summary_txt_handle.write(str_to_write + '\n')
    summary_txt_handle.close()


def metaAssembly(args):

    assembly_id_txt = args['i']
    op_dir          = args['o']
    num_threads     = args['t']
    force_overwrite = args['f']

    check_executables(['datasets', 'dataformat'])

    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()

    tmp_dir             = '%s/tmp'          % op_dir
    tmp_dir_reformatted = '%s/report'       % op_dir
    op_txt              = '%s/summary.txt'  % op_dir
    cmd_txt             = '%s/cmds.txt'     % op_dir

    os.mkdir(op_dir)
    os.mkdir(tmp_dir)

    assembly_id_set = set()
    if os.path.isfile(assembly_id_txt):
        for assembly_id in open(assembly_id_txt):
            assembly_id_set.add(assembly_id.strip().split()[0])
        if len(assembly_id_set) == 0:
            print('No id found in %s, program exited!' % assembly_id_txt)
            exit()
    else:
        assembly_id_set.add(assembly_id_txt)

    # prepare cmds for datasets and dataformat
    cmd_list = []
    cmd_txt_handle = open(cmd_txt, 'w')
    for assembly_id in assembly_id_set:
        op_metadata_tsv   = '%s/%s.txt' % (tmp_dir, assembly_id)
        ncbi_datasets_cmd = 'datasets summary genome accession %s --as-json-lines | dataformat tsv genome > %s' % (assembly_id, op_metadata_tsv)
        cmd_list.append(ncbi_datasets_cmd)
        cmd_txt_handle.write(ncbi_datasets_cmd + '\n')
    cmd_txt_handle.close()

    # run datasets and dataformat with mp
    print('Running %s commands with %s cores' % (len(cmd_list), num_threads))
    pool = mp.Pool(processes=num_threads)
    pool.map(os.system, cmd_list)
    pool.close()
    pool.join()

    # parse outputs from datasets and dataformat
    with open(op_txt, 'w') as op_txt_handle:
        op_txt_handle.write('Assembly\tBioSample\tDescription\tModel\tIsolation_Source\tMetagenome_Source\tHost\tLocation\tComment\n')

    #os.mkdir(tmp_dir_reformatted)

    print('Parsing outputs from datasets and dataformat')
    missing_output_id_set = set()
    for assembly_id in sorted(list(assembly_id_set)):
        op_metadata_tsv             = '%s/%s.txt' % (tmp_dir, assembly_id)
        op_metadata_tsv_reformatted = '%s/%s.txt' % (tmp_dir_reformatted, assembly_id)
        if os.path.isfile(op_metadata_tsv) is False:
            missing_output_id_set.add(assembly_id)

    summarise_metadata(tmp_dir, 'txt', op_txt)

    if len(missing_output_id_set) > 0:
        print('datasets and dataformat failed on the following IDs:')
        print(','.join(missing_output_id_set))

    print('Done!')


if __name__ == '__main__':

    metaAssembly_parser = argparse.ArgumentParser(usage=metaAssembly_usage)
    metaAssembly_parser.add_argument('-i', required=True,                         help='file contains assembly ids')
    metaAssembly_parser.add_argument('-o', required=True,                         help='output directory')
    metaAssembly_parser.add_argument('-f', required=False, action="store_true",   help='force overwrite')
    metaAssembly_parser.add_argument('-t', required=False, type=int, default=1,   help='number of cores, default: 1')
    args = vars(metaAssembly_parser.parse_args())
    metaAssembly(args)



import argparse
from Bio import SeqIO


rm_desc_usage = '''
====== rm_desc example commands ======

BioSAK rm_desc -f -i in.fa -o out.fa

======================================
'''


def rm_desc(args):

    fa_in   = args['i']
    fa_out  = args['o']

    op_file_handle = open(fa_out, 'w')
    for each_seq in SeqIO.parse(fa_in, 'fasta'):
        op_file_handle.write('>%s\n' % each_seq.id)
        op_file_handle.write('%s\n' % each_seq.seq)
    op_file_handle.close()


if __name__ == '__main__':

    rm_desc_parser = argparse.ArgumentParser(usage=rm_desc_usage)
    rm_desc_parser.add_argument('-i',   required=True,  help='input fasta file')
    rm_desc_parser.add_argument('-o',   required=True,  help='output fasta file')
    args = vars(rm_desc_parser.parse_args())
    rm_desc(args)



import os
import glob
import argparse
from Bio import SeqIO
import multiprocessing as mp


abd_mask_usage = '''
====================== abd_mask example commands ======================

Dependencies: metaxa2, barrnap, tRNAscan-SE, dustmasker, bedtools

BioSAK abd_mask -i gnm_dir -x fna -o mask_wd -p dRep99_406 -t 36 -f

It will:
1. rename input genomes to remove underscore
2. rename sequences in the genomes, in the format of gnm1_1, gnm1_2...   
3. remove sequences shorter than 2000 bp
4. run metaxa2, barrnap and tRNAscan-SE
5. mask the identified regions with Ns
6. The masked sequences will be exported to [prefix].masked.fna

=======================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def rename_filter_cat_gnm(gnm_dir, gnm_ext, combined_renamed_fna, rename_txt):

    gnm_file_re = '%s/*.%s' % (gnm_dir, gnm_ext)
    gnm_file_list = glob.glob(gnm_file_re)

    gnm_rename_txt_handle = open(rename_txt, 'w')
    combined_renamed_fa_handle = open(combined_renamed_fna, 'w')
    for each_gnm in sorted(gnm_file_list):
        _, _, gnm_base, _ = sep_path_basename_ext(each_gnm)
        gnm_id_no_underscore = gnm_base.replace('_', '')
        gnm_rename_txt_handle.write('%s\t%s\n' % (gnm_base, gnm_id_no_underscore))

        seq_index = 1
        for each_seq in SeqIO.parse(each_gnm, 'fasta'):
            if len(each_seq.seq) >= 2000:
                combined_renamed_fa_handle.write('>%s_%s\n%s\n' % (gnm_id_no_underscore, seq_index, str(each_seq.seq)))
                seq_index += 1
    gnm_rename_txt_handle.close()
    combined_renamed_fa_handle.close()


def abd_mask(args):

    seq_dir         = args['i']
    seq_ext         = args['x']
    op_dir          = args['o']
    op_prefix       = args['p']
    num_threads     = args['t']
    force_overwrite = args['f']

    ########################################## define file and directory name ##########################################

    renamed_combined_fna    = '%s/%s.fna'       % (op_dir, op_prefix)
    renamed_combined_txt    = '%s/%s.txt'       % (op_dir, op_prefix)
    tRNAscan_wd             = '%s/tRNAscan_wd'  % op_dir

    ####################################################################################################################

    seq_file_re     = '%s/*.%s' % (seq_dir, seq_ext)
    seq_file_list   = glob.glob(seq_file_re)
    if len(seq_file_list) == 0:
        print('No sequence files found in %s, program exited!' % seq_dir)
        exit()

    # create op dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output directory detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    # rename and combine sequences
    print('rename, filter and combine input genomes')
    rename_filter_cat_gnm(seq_dir, seq_ext, renamed_combined_fna, renamed_combined_txt)

    ####################################################################################################################

    # get rRNA regions
    shell_cmd_1_metaxa2_ssu_cmd     = 'metaxa2 --plus --mode m --cpu %s --multi_thread T --table T -g ssu --not_found T -i %s -o %s/%s.metaxa2_ssu --temp %s/metaxa2_temp_dir'  % (num_threads, renamed_combined_fna, op_dir, op_prefix, op_dir)
    shell_cmd_2_metaxa2_lsu_cmd     = 'metaxa2 --plus --mode m --cpu %s --multi_thread T --table T -g lsu --not_found T -i %s -o %s/%s.metaxa2_lsu --temp %s/metaxa2_temp_dir'  % (num_threads, renamed_combined_fna, op_dir, op_prefix, op_dir)
    shell_cmd_3                     = 'cut -f 1,9,10 %s/%s.metaxa2_ssu.extraction.results > %s/%s.masked_metaxa_ssu.bed'                                                        % (op_dir, op_prefix, op_dir, op_prefix)
    shell_cmd_4                     = 'cut -f 1,9,10 %s/%s.metaxa2_lsu.extraction.results > %s/%s.masked_metaxa_lsu.bed'                                                        % (op_dir, op_prefix, op_dir, op_prefix)
    shell_cmd_5                     = 'cat %s/%s.masked_metaxa_ssu.bed %s/%s.masked_metaxa_lsu.bed > %s/%s.masked_metaxa.bed'                                                   % (op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix)
    shell_cmd_6_barrnap_bac_cmd     = 'barrnap --kingdom bac --threads %s --reject 0.3 %s > %s/%s.barrnap_bac.gff'                                                              % (num_threads, renamed_combined_fna, op_dir, op_prefix)
    shell_cmd_7_barrnap_arc_cmd     = 'barrnap --kingdom arc --threads %s --reject 0.3 %s > %s/%s.barrnap_arc.gff'                                                              % (num_threads, renamed_combined_fna, op_dir, op_prefix)
    shell_cmd_8_barrnap_euk_cmd     = 'barrnap --kingdom euk --threads %s --reject 0.3 %s > %s/%s.barrnap_euk.gff'                                                              % (num_threads, renamed_combined_fna, op_dir, op_prefix)
    shell_cmd_9                     = 'cut -f 1,4,5 %s/%s.barrnap_bac.gff > %s/%s.masked_barrnap_bac.bed'                                                                       % (op_dir, op_prefix, op_dir, op_prefix)
    shell_cmd_10                    = 'cut -f 1,4,5 %s/%s.barrnap_arc.gff > %s/%s.masked_barrnap_arc.bed'                                                                       % (op_dir, op_prefix, op_dir, op_prefix)
    shell_cmd_11                    = 'cut -f 1,4,5 %s/%s.barrnap_euk.gff > %s/%s.masked_barrnap_euk.bed'                                                                       % (op_dir, op_prefix, op_dir, op_prefix)
    shell_cmd_12                    = 'cat %s/%s.masked_barrnap_bac.bed %s/%s.masked_barrnap_arc.bed %s/%s.masked_barrnap_euk.bed > %s/%s.masked_barrnap.bed'                   % (op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix)
    shell_cmd_13                    = 'cat %s/%s.masked_barrnap.bed %s/%s.masked_metaxa.bed > %s/%s.masked_rrna.bed'                                                            % (op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix)

    os.system('mkdir %s/metaxa2_temp_dir' % op_dir)

    print(shell_cmd_1_metaxa2_ssu_cmd)
    os.system(shell_cmd_1_metaxa2_ssu_cmd)
    print(shell_cmd_2_metaxa2_lsu_cmd)
    os.system(shell_cmd_2_metaxa2_lsu_cmd)
    print(shell_cmd_3)
    os.system(shell_cmd_3)
    print(shell_cmd_4)
    os.system(shell_cmd_4)
    print(shell_cmd_5)
    os.system(shell_cmd_5)
    print(shell_cmd_6_barrnap_bac_cmd)
    os.system(shell_cmd_6_barrnap_bac_cmd)
    print(shell_cmd_7_barrnap_arc_cmd)
    os.system(shell_cmd_7_barrnap_arc_cmd)
    print(shell_cmd_8_barrnap_euk_cmd)
    os.system(shell_cmd_8_barrnap_euk_cmd)
    print(shell_cmd_9)
    os.system(shell_cmd_9)
    print(shell_cmd_10)
    os.system(shell_cmd_10)
    print(shell_cmd_11)
    os.system(shell_cmd_11)
    print(shell_cmd_12)
    os.system(shell_cmd_12)
    print(shell_cmd_13)
    os.system(shell_cmd_13)

    ################################################# get tRNA regions #################################################

    os.system('mkdir %s' % tRNAscan_wd)

    threads_per_job = num_threads/(len(seq_file_list)*2)
    if threads_per_job == 0:
        threads_per_job = 1

    tRNAscan_cmd_list = []
    for each_gnm in seq_file_list:
        _, _, gnm_base, _ = sep_path_basename_ext(each_gnm)
        trnascan_arc_cmd = 'tRNAscan-SE -A -b %s/%s.ar.bedformat --thread %s %s'  % (tRNAscan_wd, gnm_base, threads_per_job, each_gnm)
        trnascan_bac_cmd = 'tRNAscan-SE -B -b %s/%s.bac.bedformat --thread %s %s' % (tRNAscan_wd, gnm_base, threads_per_job, each_gnm)
        tRNAscan_cmd_list.append(trnascan_arc_cmd)
        tRNAscan_cmd_list.append(trnascan_bac_cmd)

    # run tRNAscan-SE with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(os.system, tRNAscan_cmd_list)
    pool.close()
    pool.join()

    # combine the results
    shell_cmd_16 = 'cat %s/%s.ar.bedformat %s/%s.bac.bedformat > %s/%s.bedformat'   % (tRNAscan_wd, op_prefix, tRNAscan_wd, op_prefix, op_dir, op_prefix)
    shell_cmd_17 = 'cut -f 1,2,3 %s/%s.bedformat > %s/%s.masked_trnascan.bed'       % (op_dir, op_prefix, op_dir, op_prefix)
    print(shell_cmd_16)
    os.system(shell_cmd_16)
    print(shell_cmd_17)
    os.system(shell_cmd_17)

    ####################################################################################################################

    # find duplicated areas
    shell_cmd_18    = 'dustmasker -in %s -out %s/%s.lowcom.out -outfmt acclist'         % (renamed_combined_fna, op_dir, op_prefix)
    shell_cmd_19    = "cut -d '>' -f 2 %s/%s.lowcom.out > %s/%s.masked_dustmasker.bed"  % (op_dir, op_prefix, op_dir, op_prefix)
    print(shell_cmd_18)
    os.system(shell_cmd_18)
    print(shell_cmd_19)
    os.system(shell_cmd_19)

    # cover the above area with NNNN
    shell_cmd_20    = "cat %s/%s.masked_rrna.bed %s/%s.masked_trnascan.bed %s/%s.masked_dustmasker.bed > %s/%s.mask.bed"    % (op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix)
    shell_cmd_21    = "awk -F'\t' '$2>=0' %s/%s.mask.bed > %s/%s.mask_0.bed"                                                % (op_dir, op_prefix, op_dir, op_prefix)
    shell_cmd_22    = "bedtools maskfasta -fi %s -bed %s/%s.mask_0.bed -fo %s/%s.masked.fna"                                % (renamed_combined_fna, op_dir, op_prefix, op_dir, op_prefix)
    print(shell_cmd_20)
    os.system(shell_cmd_20)
    print(shell_cmd_21)
    os.system(shell_cmd_21)
    print(shell_cmd_22)
    os.system(shell_cmd_22)

    ####################################################################################################################

    print('Masked sequences exported to %s/%s.masked.fna' % (op_dir, op_prefix))
    print('Done!')


if __name__ == '__main__':

    abd_mask_parser = argparse.ArgumentParser()
    abd_mask_parser.add_argument('-i', required=True,                               help='sequence folder')
    abd_mask_parser.add_argument('-x', required=True,                               help='sequence file extension')
    abd_mask_parser.add_argument('-o', required=True,                               help='output directory')
    abd_mask_parser.add_argument('-p', required=False, default='renamed_combined',  help='output prefix, default is renamed_combined')
    abd_mask_parser.add_argument('-t', required=False, type=int, default=1,         help='number of threads, default is 1')
    abd_mask_parser.add_argument('-f', required=False, action="store_true",         help='force overwrite')
    args = vars(abd_mask_parser.parse_args())
    abd_mask(args)



import argparse
import numpy as np
from scipy.stats import wilcoxon


wilcox_usage = '''
===================== wilcox example commands ===================== 

BioSAK wilcox -1 paired_treatment_1.txt -2 paired_treatment_2.txt

===================================================================
'''


def wilcox(args):

    s1_txt = args['1']
    s2_txt = args['2']

    s1_list = []
    for e1 in open(s1_txt):
        s1_list.append(float(e1.strip()))

    s2_list = []
    for e2 in open(s2_txt):
        s2_list.append(float(e2.strip()))

    s1_array = np.array(s1_list)
    s2_array = np.array(s2_list)
    stats_result = wilcoxon(s1_array, s2_array)

    print(stats_result)


if __name__ == '__main__':

    wilcox_parser = argparse.ArgumentParser()
    wilcox_parser.add_argument('-1', required=True, help='treatment 1')
    wilcox_parser.add_argument('-2', required=True, help='treatment 2')
    args = vars(wilcox_parser.parse_args())
    wilcox(args)



import os
import glob
import argparse
import subprocess
import multiprocessing as mp


blca_usage = '''
============================== blca example commands ==============================

BioSAK blca -f -t 12 -i otu_fa -x fna -o demo -r ssu_r220.fasta -c ssu_r220.tax

# GTDB SSU (BLCA compatible)
-r /Users/songweizhi/DB/BLCA/GTDB_SSU/ssu_all_r220.blca.fa
-c /Users/songweizhi/DB/BLCA/GTDB_SSU/ssu_all_r220.blca.tax

# SILVA SSU (BLCA compatible)
-r /Users/songweizhi/DB/BLCA/SILVA_SSU/SILVA_138.2_SSURef_NR99_tax_silva.blca.fa
-c /Users/songweizhi/DB/BLCA/SILVA_SSU/SILVA_138.2_SSURef_NR99_tax_silva.blca.tax

===================================================================================
'''


def check_executables(program_list):

    not_detected_programs = []
    for needed_program in program_list:

        if subprocess.call(['which', needed_program], stdout=open(os.devnull, 'wb')) != 0:
            not_detected_programs.append(needed_program)

    if not_detected_programs != []:
        print('%s not detected, program exited!' % ','.join(not_detected_programs))
        exit()


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def parse_blca_op(blca_output):

    blca_op_name, blca_op_path, blca_op_base, blca_op_ext = sep_path_basename_ext(blca_output)
    output_file_1 = '%s/%s.1.txt' % (blca_op_path, blca_op_base)
    output_file_2 = '%s/%s.2.txt' % (blca_op_path, blca_op_base)

    # read in input file
    s16_taxon_blca_dict = {}
    for each_16s_taxon in open(blca_output):
        each_16s_taxon_split = each_16s_taxon.strip().split('\t')
        s16_taxon_blca_dict[each_16s_taxon_split[0]] = each_16s_taxon_split[1]

    taxon_dict_formatted_with_num = {}
    taxon_dict_formatted_no_num = {}
    for each_16s in s16_taxon_blca_dict:
        taxon_blca_raw = s16_taxon_blca_dict[each_16s]
        formatted_taxon_str_with_num = 'Unclassified'
        formatted_taxon_str_no_num = 'Unclassified'
        if taxon_blca_raw != 'Unclassified':
            split2 = taxon_blca_raw.strip().split(';')[:-1]
            formatted_taxon_list_with_num2 = []
            formatted_taxon_list_no_num2 = []
            element_index = 0
            while element_index < len(split2):
                taxon_blca = split2[element_index]
                confidence_value = split2[element_index + 1]
                confidence_value = float("{0:.2f}".format(float(confidence_value)))
                taxon_rank_blca = taxon_blca.split(':')[0]

                taxon_rank_gtdb = ''
                if taxon_rank_blca == 'superkingdom':
                    taxon_rank_gtdb = 'd'
                elif taxon_rank_blca == 'phylum':
                    taxon_rank_gtdb = 'p'
                elif taxon_rank_blca == 'class':
                    taxon_rank_gtdb = 'c'
                elif taxon_rank_blca == 'order':
                    taxon_rank_gtdb = 'o'
                elif taxon_rank_blca == 'family':
                    taxon_rank_gtdb = 'f'
                elif taxon_rank_blca == 'genus':
                    taxon_rank_gtdb = 'g'
                elif taxon_rank_blca == 'species':
                    taxon_rank_gtdb = 's'

                taxon_name_blca  = ':'.join(taxon_blca.split(':')[1:]).replace(':', '')
                current_rank_with_num = '%s__%s(%s)' % (taxon_rank_gtdb, taxon_name_blca, confidence_value)
                current_rank_no_num   = '%s__%s' % (taxon_rank_gtdb, taxon_name_blca)
                formatted_taxon_list_with_num2.append(current_rank_with_num)
                formatted_taxon_list_no_num2.append(current_rank_no_num)
                element_index += 2

            formatted_taxon_str_with_num = ';'.join(formatted_taxon_list_with_num2)
            formatted_taxon_str_no_num = ';'.join(formatted_taxon_list_no_num2)

        formatted_taxon_str_with_numno_space = '_'.join(formatted_taxon_str_with_num.split(' '))
        formatted_taxon_str_no_num_no_space = '_'.join(formatted_taxon_str_no_num.split(' '))

        taxon_dict_formatted_with_num[each_16s] = formatted_taxon_str_with_numno_space
        taxon_dict_formatted_no_num[each_16s] = formatted_taxon_str_no_num_no_space

    output_file_1_handle = open(output_file_1, 'w')
    output_file_2_handle = open(output_file_2, 'w')
    for each_seq in taxon_dict_formatted_with_num:
        output_file_1_handle.write('%s\t%s\n' % (each_seq, taxon_dict_formatted_with_num[each_seq]))
        output_file_2_handle.write('%s\t%s\n' % (each_seq, taxon_dict_formatted_no_num[each_seq]))
    output_file_1_handle.close()
    output_file_2_handle.close()


def blca(args):

    fa_dir              = args['i']
    fa_ext              = args['x']
    output_folder       = args['o']
    ref_seq             = args['r']
    ref_tax             = args['c']
    num_threads         = args['t']
    force_overwrite     = args['f']

    check_executables(['blastn', 'blastdbcmd', 'clustalo', 'muscle'])

    pwd_current_file    = os.path.realpath(__file__)
    current_file_path   = '/'.join(pwd_current_file.split('/')[:-1])
    blca_main_py        = '%s/blca_main.py' % (current_file_path)

    seq_file_re = '%s/*.%s' % (fa_dir, fa_ext)
    seq_file_list = glob.glob(seq_file_re)

    if len(seq_file_list) == 0:
        print('Sequence file not found, program exited')
        exit()

    # create output folder
    if os.path.isdir(output_folder) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % output_folder)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % output_folder)

    # prepare blca commands
    blca_cmd_list = []
    for seq_file in seq_file_list:
        seq_name, seq_path, seq_base, seq_ext = sep_path_basename_ext(seq_file)
        blca_cmd = 'python3 %s -q %s -r %s -p %s -i %s -o %s' % (blca_main_py, ref_seq, ref_tax, 1, seq_file, output_folder)
        print(blca_cmd)
        #os.system(blca_cmd)
        blca_cmd_list.append(blca_cmd)

    # run blca with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(os.system, blca_cmd_list)
    pool.close()
    pool.join()

    # parse blca output
    blca_op_re = '%s/*.blca.out' % output_folder
    blca_op_list = glob.glob(blca_op_re)
    for blca_op in blca_op_list:
        parse_blca_op(blca_op)

    print('Done!')


if __name__ == '__main__':

    blca_parser = argparse.ArgumentParser(usage=blca_usage)
    blca_parser.add_argument('-i',   required=True,                              help='path to input sequences (in multi-fasta format)')
    blca_parser.add_argument('-x',   required=False,                             help='file extension')
    blca_parser.add_argument('-o',   required=True,                              help='output directory')
    blca_parser.add_argument('-r',   required=True,                              help='reference sequences')
    blca_parser.add_argument('-c',   required=True,                              help='reference taxonomy')
    blca_parser.add_argument('-t',   required=False, type=int, default=1,        help='number of threads')
    blca_parser.add_argument('-f',   required=False, action="store_true",        help='force overwrite')
    args = vars(blca_parser.parse_args())
    blca(args)



import os
import glob
import argparse
import subprocess


fastaai_usage = '''
============ fastaai example commands ============

BioSAK fastaai -i faa_files -x faa -o fastaai_op

==================================================
'''

def check_executables(program_list):

    not_detected_programs = []
    for needed_program in program_list:

        if subprocess.call(['which', needed_program], stdout=open(os.devnull, 'wb')) != 0:
            not_detected_programs.append(needed_program)

    if not_detected_programs != []:
        print('%s not detected, program exited!' % ','.join(not_detected_programs))
        exit()


def fastaai(args):

    dir_in              = args['i']
    file_ext            = args['x']
    op_dir              = args['o']
    force_create_op_dir = args['f']
    num_threads         = args['t']

    check_executables(['fastaai'])

    faa_file_re   = '%s/*.%s' % (dir_in, file_ext)
    faa_file_list = glob.glob(faa_file_re)
    if len(faa_file_list) == 0:
        print('Sequence file not found, program exited!')
        exit()

    db_dir  = '%s/db'      % op_dir
    cmd_txt = '%s/cmd.txt' % op_dir

    # create op_dir
    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('output directory exist, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % db_dir)

    db_file      = '%s/database/user_genomes.db'                                                                             % db_dir
    cmd_build_db = 'fastaai build_db --proteins %s --verbose --output %s --database user_genomes.db --threads %s --compress' % (dir_in, db_dir, num_threads)
    cmd_fastaai  = 'fastaai db_query --query %s --target %s --verbose --output %s --threads %s --output_style matrix'        % (db_file, db_file, op_dir, num_threads)

    # write out commands
    cmd_txt_handle = open(cmd_txt, 'w')
    cmd_txt_handle.write(cmd_build_db + '\n')
    cmd_txt_handle.write(cmd_fastaai + '\n')
    cmd_txt_handle.close()
    print('Commands exported to %s' % cmd_txt)

    # run commands
    os.system(cmd_build_db)
    os.system(cmd_fastaai)

    print('Results exported to %s/results/FastAAI_matrix.txt' % op_dir)
    print('Done!')


if __name__ == '__main__':

    fastaai_parser = argparse.ArgumentParser(usage=fastaai_usage)
    fastaai_parser.add_argument('-i', required=True,                          help='faa files')
    fastaai_parser.add_argument('-x', required=False, default='faa',          help='file extension, default: faa')
    fastaai_parser.add_argument('-o', required=True,                          help='output directory')
    fastaai_parser.add_argument('-t', required=False, default=1, type=int,    help='number of threads, default: 1')
    fastaai_parser.add_argument('-f', required=False, action="store_true",    help='force overwrite')
    args = vars(fastaai_parser.parse_args())
    fastaai(args)



import os
import argparse
from Bio import SeqIO


rename_reads_for_Reago_usage = '''
=========================== rename_reads_for_Reago example commands ===========================

BioSAK rename_reads_for_Reago -in Soil_R1.fasta -out Soil_R1_renamed.fasta -p Soil -d 1
BioSAK rename_reads_for_Reago -in Soil_R2.fasta -out Soil_R2_renamed.fasta -p Soil -d 2

# Note
1. The order of paired reads in the two paired reads files must be the same.
2. Prefix of renamed reads (-p) for the two paired reads files must be the same.

===============================================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '../My_Python_scripts/scripts_backup'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def rename_reads_for_Reago(args):

    input_file   = args['in']
    output_file  = args['out']
    reads_prefix = args['p']
    direction    = args['d']

    output_file_handle = open(output_file, 'w')
    seq_index = 1
    for seq_record in SeqIO.parse(input_file, 'fasta'):
        output_file_handle.write('>%s_%s.%s\n' % (reads_prefix, seq_index, direction))
        output_file_handle.write('%s\n' % seq_record.seq)
        seq_index += 1

    output_file_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='', add_help=False)

    parser.add_argument('-in',  required=True,  type=str, help='input fasta file')
    parser.add_argument('-out', required=True,  type=str, help='renamed fasta file')
    parser.add_argument('-p',   required=True,  type=str, help='prefix of renamed reads')
    parser.add_argument('-d',   required=True,  type=int, help='chose from 1 (forward) or 2 (reverse)')

    args = vars(parser.parse_args())

    rename_reads_for_Reago(args)




import os
import argparse

get_genome_GTDB_parser_usage = '''
================================== get_genome_GTDB example commands ==================================

# Example commands
BioSAK get_genome_GTDB -i genome.txt -fastani_dir release214/fastani -o op_gnm_folder -force

# -fastani_dir: normally located in folder named as "release214"

# example of genome id file (one id per line):
GCA_902608175.1
GCA_903901015.1
GCF_000007225.1

=====================================================================================================
'''


def get_genome_GTDB(args):

    gnm_id_txt          = args['i']
    gtdb_ref_gnm_folder = args['fastani_dir']
    op_gnm_folder       = args['o']
    force_overwrite     = args['force']

    gtdb_ref_gnm_path_txt = '%s/genome_paths.tsv' % gtdb_ref_gnm_folder
    if os.path.isfile(gtdb_ref_gnm_path_txt) is False:
        print('genome_paths.tsv not found in %s, program exited!' % gtdb_ref_gnm_folder)
        exit()

    if os.path.isdir(op_gnm_folder) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_gnm_folder)
        else:
            print('%s detected, program exited!' % op_gnm_folder)
            exit()
    os.mkdir(op_gnm_folder)

    gnms_not_found_txt = '%s_missing_genomes.txt' % op_gnm_folder

    # read in gtdb_ref_gnm_path_txt
    ref_gnm_to_path_dict = {}
    for each_ref_gnm in open(gtdb_ref_gnm_path_txt):
        ref_gnm_split = each_ref_gnm.strip().split(' ')
        ref_accession = ref_gnm_split[0].split('_genomic')[0]
        pwd_ref_gnm = '%s/%s%s' % (gtdb_ref_gnm_folder, ref_gnm_split[1], ref_gnm_split[0])
        ref_gnm_to_path_dict[ref_accession] = pwd_ref_gnm

    missing_gnm_set = set()
    for each_gnm in open(gnm_id_txt):

        if not each_gnm.startswith('Genome\tCompleteness\tContamination'):

            # get gnm_id
            if '\t' in each_gnm:
                gnm_id = each_gnm.strip().split('\t')[0]
            elif ' ' in each_gnm:
                gnm_id = each_gnm.strip().split(' ')[0]
            else:
                gnm_id = each_gnm.strip()

            if gnm_id in ref_gnm_to_path_dict:
                pwd_gnm = ref_gnm_to_path_dict[gnm_id]
                cmd_cp = 'cp %s %s/' % (pwd_gnm, op_gnm_folder)
                cmd_gunzip = 'gunzip %s/%s_genomic.fna.gz' % (op_gnm_folder, gnm_id)
                cmd_rename = 'mv %s/%s_genomic.fna %s/%s.fna' % (op_gnm_folder, gnm_id, op_gnm_folder, gnm_id)
                os.system(cmd_cp)
                os.system(cmd_gunzip)
                os.system(cmd_rename)
            else:
                missing_gnm_set.add(gnm_id)

    if len(missing_gnm_set) > 0:
        print('Genomes not found in GTDB: %s, see details in %s' % (len(missing_gnm_set), gnms_not_found_txt))
        gnms_not_found_txt_handle = open(gnms_not_found_txt, 'w')
        for each_unfound_gnm in missing_gnm_set:
            gnms_not_found_txt_handle.write('%s\n' % each_unfound_gnm)
        gnms_not_found_txt_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser(usage=get_genome_GTDB_parser_usage)
    parser.add_argument('-i',           required=True,                          help='genome id')
    parser.add_argument('-o',           required=True,                          help='output folder')
    parser.add_argument('-fastani_dir', required=True,                          help='the "fastani" dir')
    parser.add_argument('-force',       required=False, action="store_true",    help='Force overwrite existing results')
    args = vars(parser.parse_args())
    get_genome_GTDB(args)



import os
import argparse


metabat2maxbin_usage = '''
=============== metabat2maxbin example commands ===============

BioSAK metabat2maxbin -p maxbin2_depth -i metabat_depth.txt

===============================================================
'''

def metabat2maxbin(args):

    maxbin_depth_prefix = args['p']
    metabat_depth_txt   = args['i']

    to_write_dict = dict()
    line_index = 1
    for each_line in open(metabat_depth_txt):
        each_line_split = each_line.strip().split('\t')

        col_index = 0
        sample_index = 1
        for each_col in each_line_split:
            if (col_index >= 3) and ((col_index % 2) != 0):

                op_txt = '%s_%s.txt' % (maxbin_depth_prefix, sample_index)
                if op_txt not in to_write_dict:
                    to_write_dict[op_txt] = []

                if line_index >= 2:  # to ignore the header line
                    to_write_dict[op_txt].append('%s\t%s' % (each_line_split[0], each_col))

                sample_index += 1
            col_index += 1
        line_index += 1

    for each_file in to_write_dict:
        file_handle = open(each_file, 'w')
        file_handle.write('\n'.join(to_write_dict[each_file]))
        file_handle.close()


if __name__ == '__main__':

    metabat2maxbin_parser = argparse.ArgumentParser(usage=metabat2maxbin_usage)
    metabat2maxbin_parser.add_argument('-p',   required=True,  help='output prefix for maxbin depth file')
    metabat2maxbin_parser.add_argument('-i',   required=True,  help='metabat depth')
    args = vars(metabat2maxbin_parser.parse_args())
    metabat2maxbin(args)



import os
import math
import argparse


js_hpc3_usage = '''
==================== js_hpc3 example commands ====================

BioSAK js_hpc3 -p Demo -c cmds.txt -hd js_header.sh -n 3 -force

==================================================================
'''


def js_hpc3(args):

    js_prefix       = args['p']
    cmd_file        = args['c']
    js_header_file  = args['hd']
    cmd_num_per_js  = args['n']
    force_overwrite = args['f']
    cmd_t           = args['t']

    pwd_js_folder = '%s_job_scripts' % js_prefix

    if (os.path.isdir(pwd_js_folder) is True) and (force_overwrite is False):
        print('Output folder detected, program exited: %s' % pwd_js_folder)
        exit()
    else:
        if os.path.isdir(pwd_js_folder) is True:
            os.system('rm -r %s' % pwd_js_folder)
            os.mkdir(pwd_js_folder)
        else:
            os.mkdir(pwd_js_folder)

    # read in js header
    js_header_str = ''
    for each_line in open(js_header_file):
        js_header_str += each_line

    # get the total number of commands
    cmd_num = 0
    for each_cmd in open(cmd_file):
        if len(each_cmd.strip()) > 0:
            cmd_num += 1

    n = 1
    for each_cmd in open(cmd_file):
        if len(each_cmd.strip()) > 0:
            js_index = math.ceil((n-1)//cmd_num_per_js) + 1
            pwd_js = '%s/%s_%s.sh' % (pwd_js_folder, js_prefix, js_index)
            if os.path.isfile(pwd_js) is False:
                pwd_js_handle = open(pwd_js, 'w')
                pwd_js_handle.write(js_header_str)
                pwd_js_handle.write('\n')
                if n < cmd_num:
                    pwd_js_handle.write('srun -n %s %s &\n' % (cmd_t, each_cmd.strip()))
                else:
                    pwd_js_handle.write('%s\n' % each_cmd.strip())
                pwd_js_handle.close()
            else:
                pwd_js_handle = open(pwd_js, 'a')
                if (math.ceil((n-1)//cmd_num_per_js) + 1) == (math.ceil((n)//cmd_num_per_js) + 1):
                    if n < cmd_num:
                        pwd_js_handle.write('srun -n %s %s &\n' % (cmd_t, each_cmd.strip()))
                    else:
                        pwd_js_handle.write('srun -n %s %s\n' % (cmd_t, each_cmd.strip()))
                        pwd_js_handle.write('wait\n')
                else:
                    pwd_js_handle.write('srun -n %s %s\n' % (cmd_t, each_cmd.strip()))
                    pwd_js_handle.write('wait\n')
                pwd_js_handle.close()
            n += 1


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-p',       required=True,                          help='js prefix')
    parser.add_argument('-c',       required=True,                          help='cmds file')
    parser.add_argument('-hd',      required=True,                          help='js header')
    parser.add_argument('-n',       required=True,  type=int,               help='number of cmds per job script')
    parser.add_argument('-t',       required=False, type=int, default=1,    help='number of threads specified in the commands')
    parser.add_argument('-f',       required=False, action="store_true",    help='force overwrite existing results')
    args = vars(parser.parse_args())
    js_hpc3(args)



import os
import argparse


sampling_GTDB_gnms_usage = '''
====================================== sampling_GTDB_gnms example commands ======================================

# Example commands
BioSAK sampling_GTDB_gnms -o Thermo_o.txt -meta ar53_metadata_r214.tsv -taxon p__Thermoproteota -r o
BioSAK sampling_GTDB_gnms -o Thermo_f.txt -meta ar53_metadata_r214.tsv -taxon p__Thermoproteota -r f -cpl 85 -ctm 5 -rs
BioSAK sampling_GTDB_gnms -o Firm_g.txt -meta bac120_metadata_r214.tsv -taxon p__Firmicutes -r g -cpl 85 -ctm 5 -ts

# -meta: ar53_metadata_r214.tsv or bac120_metadata_r214.tsv
# Sampling rank need to be lower than the rank of specified taxon (-taxon).
# Genome(s) with highest quality score(s)(defined as completeness-5*contamination) will be selected.

=================================================================================================================
'''


def sampling_GTDB_gnms(args):

    output_table            = args['o']
    gtdb_gnm_metadata       = args['meta']
    interested_taxon        = args['taxon']
    sampling_rank           = args['r']
    gnm_num_per_taxon       = args['n']
    cpl_cutoff              = args['cpl']
    ctm_cutoff              = args['ctm']
    type_strain_only        = args['ts']
    representative_species  = args['rs']

    dod = {}
    gnm_cpl_dict = {}
    gnm_ctm_dict = {}
    gnm_size_dict = {}
    gnm_metric_dict = {}
    gnm_to_taxon_dict = {}
    col_index = {}
    for each_ref in open(gtdb_gnm_metadata):
        each_ref_split = each_ref.strip().split('\t')
        if each_ref.startswith('accession	ambiguous_bases'):
            col_index = {key: i for i, key in enumerate(each_ref_split)}
        else:
            ref_accession = each_ref_split[0][3:]
            gnm_completeness = float(each_ref_split[2])
            gnm_contamination = float(each_ref_split[3])
            gnm_size = each_ref_split[col_index['genome_size']]
            gtdb_taxonomy = each_ref_split[col_index['gtdb_taxonomy']]

            try:
                is_type = each_ref_split[col_index['gtdb_type_designation']]
            except:
                is_type = each_ref_split[col_index['gtdb_type_designation_ncbi_taxa']]

            is_representative = each_ref_split[col_index['gtdb_representative']]

            if interested_taxon in gtdb_taxonomy:

                # check type
                type_pass = False
                if type_strain_only is False:
                    type_pass = True
                else:
                    if is_type in ['type strain of species', 'type strain of heterotypic synonym']:
                        type_pass = True

                # check gtdb_representative
                representative_pass = False
                if representative_species is False:
                    representative_pass = True
                else:
                    if is_representative == 't':
                        representative_pass = True

                # check completeness
                cpl_passed = False
                if cpl_cutoff is None:
                    cpl_passed = True
                else:
                    if gnm_completeness >= float(cpl_cutoff):
                        cpl_passed = True

                # check contamination
                ctm_passed = False
                if ctm_cutoff is None:
                    ctm_passed = True
                else:
                    if gnm_contamination <= float(ctm_cutoff):
                        ctm_passed = True

                if (type_pass is True) and (representative_pass is True) and (cpl_passed is True) and (ctm_passed is True):

                    # add to dict
                    gnm_to_taxon_dict[ref_accession] = gtdb_taxonomy
                    gnm_cpl_dict[ref_accession] = gnm_completeness
                    gnm_ctm_dict[ref_accession] = gnm_contamination
                    gnm_size_dict[ref_accession] = gnm_size
                    gnm_quality_metric = gnm_completeness - (5 * gnm_contamination)
                    gnm_metric_dict[ref_accession] = float("{0:.3f}".format(gnm_quality_metric))

                    # get taxon at sampling rank
                    sampling_rank_taxon = ''
                    for each_rank_taxon in gtdb_taxonomy.split(';'):
                        if each_rank_taxon[:3] == '%s__' % sampling_rank:
                            sampling_rank_taxon = each_rank_taxon

                    if sampling_rank_taxon not in dod:
                        dod[sampling_rank_taxon] = dict()

                    dod[sampling_rank_taxon][ref_accession] = gnm_quality_metric

    # write out selection
    selected_gnm_meta_txt_handle = open(output_table, 'w')
    selected_gnm_meta_txt_handle.write('Genome\tCompleteness\tContamination\tSelection_criterion(completeness-5*contamination)\tSize(bp)\tTaxonomy\n')
    selected_gnm_list = []
    for each_sampling_taxon in dod:
        current_taxon_gnm_quality_dict = dod[each_sampling_taxon]
        gnms_sorted_by_quality = sorted(current_taxon_gnm_quality_dict.items(), key=lambda x: x[1], reverse=True)
        selected_gnm = 0
        for each_gnm in gnms_sorted_by_quality:
            if selected_gnm < gnm_num_per_taxon:
                gnm_id = each_gnm[0]
                selected_gnm_list.append(gnm_id)
                str_to_write = '%s\t%s\t%s\t%s\t%s\t%s' % (gnm_id, gnm_cpl_dict[gnm_id], gnm_ctm_dict[gnm_id], gnm_metric_dict[gnm_id], gnm_size_dict[gnm_id], gnm_to_taxon_dict[gnm_id])
                selected_gnm_meta_txt_handle.write(str_to_write + '\n')
                selected_gnm += 1
    selected_gnm_meta_txt_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-o',           required=True,                              help='output table')
    parser.add_argument('-meta',        required=True,                              help='GTDB reference genome metadata')
    parser.add_argument('-taxon',       required=True,                              help='interested taxon')
    parser.add_argument('-r',           required=True,                              help='sampling at rank, select from p, c, o, f, g and s')
    parser.add_argument('-n',           required=False, default=1, type=int,        help='numer of genome to retain per taxon')
    parser.add_argument('-cpl',         required=False, default=None, type=float,   help='completeness cutoff (0-100), default: None')
    parser.add_argument('-ctm',         required=False, default=None, type=float,   help='contamination cutoff, default: None')
    parser.add_argument('-ts',          required=False, action='store_true',        help='only consider type strain')
    parser.add_argument('-rs',          required=False, action='store_true',        help='only consider representative species')
    args = vars(parser.parse_args())
    sampling_GTDB_gnms(args)



import os
import argparse
import certifi
os.environ["SSL_CERT_FILE"] = certifi.where()
os.environ["REQUESTS_CA_BUNDLE"] = certifi.where()
from ete3 import NCBITaxa


parse_mmseqs_tsv_usage = '''
============ parse_mmseqs_tsv example command ============

BioSAK parse_mmseqs_tsv -i taxonomyResult.tsv -o out.txt

==========================================================
'''


def parse_mmseqs_tsv(args):

    mmseqs_tsv_in   = args['i']
    table_out       = args['o']

    ncbi = NCBITaxa()

    table_out_handle = open(table_out, 'w')
    for each_line in open(mmseqs_tsv_in):
        each_line_split = each_line.strip().split('\t')
        seq_id = each_line_split[0]
        tax_id = each_line_split[1]

        name_str = 'na'
        if tax_id not in ['0']:
            lineage = ncbi.get_lineage(int(tax_id))
            name_dict = ncbi.get_taxid_translator(lineage)
            name_list = [name_dict[i] for i in lineage]
            name_str = ','.join(name_list)

        table_out_handle.write('%s\t%s\n' % (seq_id, name_str))
    table_out_handle.close()


if __name__ == '__main__':

    parse_mmseqs_tsv_parser = argparse.ArgumentParser(usage=parse_mmseqs_tsv_usage)
    parse_mmseqs_tsv_parser.add_argument('-i',  required=True, help='tsv file produced by "mmseqs createtsv"')
    parse_mmseqs_tsv_parser.add_argument('-o',  required=True, help='output table')
    args = vars(parse_mmseqs_tsv_parser.parse_args())
    parse_mmseqs_tsv(args)



import os
import glob
import argparse


FasterqDump_usage = '''
========== FasterqDump example commands ==========

BioSAK FasterqDump -i sra_dir -o op_dir -gzip -f

==================================================
'''

def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def FasterqDump(args):

    sra_dir             = args['i']
    file_ext            = args['x']
    op_dir              = args['o']
    run_gzip            = args['gzip']
    force_create_op_dir = args['f']

    fasterq_dump_tmp    = '%s/tmp'                      % op_dir
    cmd_txt             = '%s/fasterq-dump_cmds.txt'    % op_dir

    sra_file_re = '%s/*.%s' % (sra_dir, file_ext)
    sra_file_list = glob.glob(sra_file_re)

    if len(sra_file_list) == 0:
        print('Input sra file not found, program exited!')
        exit()

    # create output folder
    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % fasterq_dump_tmp)

    fasterq_dump_cmd_list = []
    cmd_txt_handle = open(cmd_txt, 'w')
    for each_sra in sra_file_list:
        _, _, sra_base, _ = sep_path_basename_ext(each_sra)
        if run_gzip is False:
            fasterq_dump_cmd = 'fasterq-dump %s --split-3 -O %s -t %s' % (each_sra, sra_base, fasterq_dump_tmp)
        else:
            fasterq_dump_cmd = 'fasterq-dump %s --split-3 -O %s -t %s; gzip %s/%s_1.fastq; gzip %s/%s_2.fastq' % (each_sra, op_dir, fasterq_dump_tmp, op_dir, sra_base, op_dir, sra_base)
        cmd_txt_handle.write(fasterq_dump_cmd + '\n')
        fasterq_dump_cmd_list.append(fasterq_dump_cmd)
    cmd_txt_handle.close()

    # run fasterq-dump
    for each_cmd in fasterq_dump_cmd_list:
        print('Processing %s' % each_cmd.split(' ')[1])
        os.system(each_cmd)

    print('Done!')


if __name__ == '__main__':

    FasterqDump_parser = argparse.ArgumentParser()
    FasterqDump_parser.add_argument('-i',       required=True,                          help='sra directory')
    FasterqDump_parser.add_argument('-x',       required=False,default='sra',           help='sra directory, default is sra')
    FasterqDump_parser.add_argument('-o',       required=True,                          help='output directory')
    FasterqDump_parser.add_argument('-f',       required=False, action="store_true",    help='force overwrite')
    FasterqDump_parser.add_argument('-gzip',    required=False, action="store_true",    help='compress with gzip')
    args = vars(FasterqDump_parser.parse_args())
    FasterqDump(args)



import argparse
from Bio import SeqIO


rm_short_usage = '''
========= rm_short example commands =========

BioSAK rm_short -f -i in.fa -o out.fa -l 50

=============================================
'''


def rm_short(args):

    fa_in   = args['i']
    fa_out  = args['o']
    min_len = args['l']

    op_file_handle = open(fa_out, 'w')
    for each_seq in SeqIO.parse(fa_in, 'fasta'):
        if len(each_seq.seq) >= min_len:
            op_file_handle.write('>%s\n' % each_seq.id)
            op_file_handle.write('%s\n' % each_seq.seq)
    op_file_handle.close()


if __name__ == '__main__':

    rm_short_parser = argparse.ArgumentParser(usage=rm_short_usage)
    rm_short_parser.add_argument('-i',   required=True,             help='input fasta file')
    rm_short_parser.add_argument('-l',   required=True,type=int,    help='length cutoff in bp')
    rm_short_parser.add_argument('-o',   required=True,             help='output fasta file')
    args = vars(rm_short_parser.parse_args())
    rm_short(args)



import os
from setuptools import setup


def version():

    setup_dir = os.path.dirname(os.path.realpath(__file__))
    version_file = open(os.path.join(setup_dir, 'BioSAK', 'VERSION'))
    return version_file.readline().strip()


__long_description__ = ''' BioSAK v%s ''' % version()


setup(name="biosak",
      version=version(),
      long_description=__long_description__,
      license="GPL3+",
      author="Weizhi Song",
      author_email="songwz03@gmail.com",
      keywords="Bioinformatics",
      description="BioSAK",
      url="https://github.com/songweizhi/BioSAK",
      packages=['BioSAK'],
      package_data={'': ['*.r', '*.R', '*.py', '*.pl', 'VERSION', '*.hmm']},
      include_package_data=True,
      install_requires=['biopython', 'matplotlib', 'numpy', 'scipy', 'itolapi', 'networkx', 'seaborn', 'lxml', 'beautifulsoup4', 'ete3', 'statsmodels'],  # reportlab
      scripts=['bin/BioSAK'])




import argparse


DendroDF_usage = '''
================== DendroDF example commands ==================

BioSAK DendroDF -i matrix.txt -o Dendrogram.pdf

===============================================================
'''


def DendroDF(args):

    input_matrix_txt = args['i']
    output_pdf_txt   = args['o']


if __name__ == '__main__':

    DendroDF_parser = argparse.ArgumentParser(usage=DendroDF_usage)
    DendroDF_parser.add_argument('-i', required=True, help='input data matrix')
    DendroDF_parser.add_argument('-o', required=True, help='output plot, in pdf format')
    args = vars(DendroDF_parser.parse_args())
    DendroDF(args)



import argparse
import pandas as pd


rm_low_depth_sample_usage = '''
===================== rm_low_depth_sample example commands =====================

BioSAK rm_low_depth_sample -i otu_table.txt -c 20000 -o otu_table_mim20000.txt

================================================================================
'''


def rm_low_depth_sample(args):

    otu_table_in            = args['i']
    abd_cutoff              = args['c']
    otu_table_out           = args['o']
    otu_table_df            = pd.read_csv(otu_table_in, sep='\t', header=0, index_col=0)
    column_sums             = otu_table_df.sum()
    column_sum_dict         = column_sums.to_dict()
    otu_table_df_filtered   = otu_table_df.loc[:, column_sums >= abd_cutoff]
    otu_table_df_filtered.to_csv(otu_table_out, sep='\t')

    # report
    print('The following samples were removed from the OTU table:')
    for sample in sorted(column_sum_dict.keys()):
        seq_count = column_sum_dict[sample]
        if seq_count < abd_cutoff:
            print('%s\t%s' % (sample, seq_count))
    print('Done!')


if __name__ == '__main__':

    rm_low_depth_sample_parser = argparse.ArgumentParser()
    rm_low_depth_sample_parser.add_argument('-i', required=True,                            help='input otu count table')
    rm_low_depth_sample_parser.add_argument('-c', required=False, default=20000,type=int,   help='minimal number of sequences, default is 20000')
    rm_low_depth_sample_parser.add_argument('-o', required=True,                            help='output otu count table')
    args = vars(rm_low_depth_sample_parser.parse_args())
    rm_low_depth_sample(args)



import os
import argparse


rename_df_row_usage = '''
======================== rename_df_row example command ========================

BioSAK rename_df_row -i df_in.txt -r rename.txt -o df_out.txt
BioSAK rename_df_row -i df_in.txt -r rename.txt -o df_out.txt -skip_top_row

Note: dataframe need to be tab separated.

===============================================================================
'''


def rename_df_row(args):

    df_in           = args['i']
    rename_txt      = args['r']
    df_out          = args['o']
    skip_top_row    = args['skip_top_row']

    # check input file
    if rename_txt is not None:
        if os.path.isfile(df_in) is False:
            print('%s not found, program exited!' % df_in)
            exit()

    if rename_txt is not None:
        if os.path.isfile(rename_txt) is False:
            print('%s not found, program exited!' % rename_txt)
            exit()

    # get genome rename dict
    rename_dict = dict()
    for line in open(rename_txt):
        line_split = line.strip().split()
        rename_dict[line_split[0]] = line_split[1]

    # rename row header
    line_index = 1
    df_out_handle = open(df_out, 'w')
    for each_line in open(df_in):
        each_line_split = each_line.strip().split('\t')
        row_header      = each_line_split[0]
        row_header_new  = rename_dict.get(row_header, row_header)
        if line_index == 1:
            if skip_top_row is True:
                df_out_handle.write(each_line)
            else:
                df_out_handle.write(each_line.replace(row_header, row_header_new))
        else:
            df_out_handle.write(each_line.replace(row_header, row_header_new))
        line_index += 1
    df_out_handle.close()


if __name__ == '__main__':

    rename_df_row_parser = argparse.ArgumentParser(usage=rename_df_row_usage)
    rename_df_row_parser.add_argument('-i',             required=True,                        help='input dataframe')
    rename_df_row_parser.add_argument('-r',             required=True,                        help='rename file, tab separated')
    rename_df_row_parser.add_argument('-o',             required=True,                        help='output dataframe')
    rename_df_row_parser.add_argument('-skip_top_row',  required=False, action="store_true",  help='skip the first row')
    args = vars(rename_df_row_parser.parse_args())
    rename_df_row(args)



import os
import argparse


stats_ko_usage = '''
=================================== stats_ko example commands ===================================

BioSAK stats_ko -ko ko_d.txt -db ko00001.keg -o op_dir -e A09150,A09160,09144,03008

# Required DB files:
ko00001.keg: https://www.genome.jp/kegg-bin/download_htext?htext=ko00001&format=htext&filedir=

A09150	Organismal Systems
A09160  Human Diseases
09144 Cellular community - eukaryotes
03008 Ribosome biogenesis in eukaryotes
99980 Enzymes with EC numbers

=================================================================================================
'''


def ko_stats_to_txt(ko_stats_dict, ko_desc_dict, op_txt, op_stats_txt):

    op_txt_handle = open(op_txt, 'w')
    op_stats_txt_handle = open(op_stats_txt, 'w')
    for ko_high in sorted(list(ko_stats_dict.keys())):
        ko_d_set = ko_stats_dict[ko_high]
        ko_high_desc = ko_desc_dict[ko_high]
        op_stats_txt_handle.write('%s\t%s\t%s\n' % (ko_high, len(ko_d_set), ko_high_desc))
        for ko_d in sorted(list(ko_d_set)):
            ko_d_desc = ko_desc_dict[ko_d]
            op_txt_handle.write('%s\t%s\t%s\t%s\t%s\n' % (ko_high, len(ko_d_set), ko_high_desc.strip(), ko_d, ko_d_desc.strip()))
    op_txt_handle.close()
    op_stats_txt_handle.close()


def stats_ko(args):

    ko_txt                  = args['ko']
    op_dir                  = args['o']
    db_file                 = args['db']
    force_create_op_dir     = args['f']
    specified_ids_to_ignore = args['e']

    specified_ids_to_ignore_set = specified_ids_to_ignore.split(',')

    ####################################################################################################################

    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    op_a_txt        = '%s/ko_A.txt'       % op_dir
    op_a_stats_txt  = '%s/ko_A_stats.txt' % op_dir
    op_b_txt        = '%s/ko_B.txt'       % op_dir
    op_b_stats_txt  = '%s/ko_B_stats.txt' % op_dir
    op_c_txt        = '%s/ko_C.txt'       % op_dir
    op_c_stats_txt  = '%s/ko_C_stats.txt' % op_dir

    ############################################## Read in KEGG DB files ###############################################

    print('Read in KEGG DB files')
    As_description_dict = {}
    Bs_description_dict = {}
    Cs_description_dict = {}
    Ds_description_dict = {}
    D2ABCD_dict = {}
    current_A = ''
    current_B = ''
    current_C = ''
    for each_line in open(db_file):
        if each_line[0] in ['A', 'B', 'C', 'D']:
            each_line_split = each_line.strip().split(' ')
            if each_line[0] == 'A':
                current_A_id = each_line_split[0]
                current_A_description = ' '.join(each_line_split[1:])
                current_A = current_A_id
                As_description_dict[current_A_id] = current_A_description
            elif each_line[0] == 'B':
                if len(each_line_split) > 1:
                    current_B_id = each_line_split[2]
                    current_B_description = ' '.join(each_line_split[3:])
                    current_B = current_B_id
                    Bs_description_dict[current_B_id] = current_B_description
            elif each_line[0] == 'C':
                current_C_id = each_line_split[4]
                current_C_description = ' '.join(each_line_split[5:])
                current_C = current_C_id
                Cs_description_dict[current_C_id] = current_C_description
            elif each_line[0] == 'D':
                current_D_id = each_line_split[6]
                current_D_description = ' '.join(each_line_split[7:])
                Ds_description_dict[current_D_id] = current_D_description
                ABCD_value = 'A_%s|B_%s|C_%s|D_%s' % (current_A, current_B, current_C, current_D_id)
                if current_D_id not in D2ABCD_dict:
                    D2ABCD_dict[current_D_id] = [ABCD_value]
                elif (current_D_id in D2ABCD_dict) and (ABCD_value not in D2ABCD_dict[current_D_id]):
                    D2ABCD_dict[current_D_id].append(ABCD_value)

    ABCD_description_dict = {}
    for each_a in As_description_dict:
        ABCD_description_dict[each_a] = As_description_dict[each_a]
    for each_b in Bs_description_dict:
        ABCD_description_dict[each_b] = Bs_description_dict[each_b]
    for each_c in Cs_description_dict:
        ABCD_description_dict[each_c] = Cs_description_dict[each_c]
    for each_d in Ds_description_dict:
        ABCD_description_dict[each_d] = Ds_description_dict[each_d]

    to_ignore_ABC_set = set()
    for d in D2ABCD_dict:
        for each in D2ABCD_dict[d]:
            each_split = each.split('|')
            id_a = each_split[0][2:]
            id_b = each_split[1][2:]
            id_c = each_split[2][2:]
            if id_a in specified_ids_to_ignore_set:
                to_ignore_ABC_set.add(id_a)
                to_ignore_ABC_set.add(id_b)
                to_ignore_ABC_set.add(id_c)
            if id_b in specified_ids_to_ignore_set:
                to_ignore_ABC_set.add(id_b)
                to_ignore_ABC_set.add(id_c)
            if id_c in specified_ids_to_ignore_set:
                to_ignore_ABC_set.add(id_c)

    ########################################################################################################################

    input_ko_id_set = set()
    for each_ko in open(ko_txt):
        ko_id = each_ko.strip().split()[0]
        if ko_id.startswith('K'):
            input_ko_id_set.add(ko_id)

    ko_a_stats_dict = dict()
    ko_b_stats_dict = dict()
    ko_c_stats_dict = dict()
    for ko_id in input_ko_id_set:
        ko_abcd_list = D2ABCD_dict.get(ko_id, [])
        for ko_abcd in ko_abcd_list:
            ko_abcd_split = ko_abcd.split('|')
            ko_a = ko_abcd_split[0][2:]
            ko_b = ko_abcd_split[1][2:]
            ko_c = ko_abcd_split[2][2:]

            # ko_a
            if ko_a not in to_ignore_ABC_set:
                if ko_a not in ko_a_stats_dict:
                    ko_a_stats_dict[ko_a] = set()
                ko_a_stats_dict[ko_a].add(ko_id)

            # ko_b
            if ko_b not in to_ignore_ABC_set:
                if ko_b not in ko_b_stats_dict:
                    ko_b_stats_dict[ko_b] = set()
                ko_b_stats_dict[ko_b].add(ko_id)

            # ko_c
            if ko_c not in to_ignore_ABC_set:
                if ko_c not in ko_c_stats_dict:
                    ko_c_stats_dict[ko_c] = set()
                ko_c_stats_dict[ko_c].add(ko_id)

    ko_stats_to_txt(ko_a_stats_dict, ABCD_description_dict, op_a_txt, op_a_stats_txt)
    ko_stats_to_txt(ko_b_stats_dict, ABCD_description_dict, op_b_txt, op_b_stats_txt)
    ko_stats_to_txt(ko_c_stats_dict, ABCD_description_dict, op_c_txt, op_c_stats_txt)

    print('Done!')


if __name__ == "__main__":

    stats_ko_parser = argparse.ArgumentParser(usage=stats_ko_usage)
    stats_ko_parser.add_argument('-ko', required=True,                          help='ko.txt')
    stats_ko_parser.add_argument('-db', required=True,                          help='ko00001.keg')
    stats_ko_parser.add_argument('-o',  required=True,                          help='output directory')
    stats_ko_parser.add_argument('-e',  required=False, default='',             help='IDs to exclude, at ABC levels')
    stats_ko_parser.add_argument('-f',  required=False, action="store_true",    help='force overwrite')
    args = vars(stats_ko_parser.parse_args())
    stats_ko(args)
